class PropertyWithSetter:
    @property
    def foo(self) -> str:
        """Docstring for foo."""
        return "foo"

    @foo.setter
    def foo(self, value: str) -> None:
        pass

    @foo.deleter
    def foo(self):
        pass

    @foo
    def foo(self, value: str) -> None:
        pass



import pytest


@pytest.mark.parametrize("param", (1, 2))
def test_tuple(param):
    ...


@pytest.mark.parametrize(
    ("param1", "param2"),
    (
        (1, 2),
        (3, 4),
    ),
)
def test_tuple_of_tuples(param1, param2):
    ...


@pytest.mark.parametrize(
    ("param1", "param2"),
    (
        [1, 2],
        [3, 4],
    ),
)
def test_tuple_of_lists(param1, param2):
    ...


@pytest.mark.parametrize("param", [1, 2])
def test_list(param):
    ...


@pytest.mark.parametrize(
    ("param1", "param2"),
    [
        (1, 2),
        (3, 4),
    ],
)
def test_list_of_tuples(param1, param2):
    ...


@pytest.mark.parametrize(
    ("param1", "param2"),
    [
        [1, 2],
        [3, 4],
    ],
)
def test_list_of_lists(param1, param2):
    ...


@pytest.mark.parametrize(
    "param1,param2",
    [
        [1, 2],
        [3, 4],
    ],
)
def test_csv_name_list_of_lists(param1, param2):
    ...


@pytest.mark.parametrize(
    "param",
    [
        [1, 2],
        [3, 4],
    ],
)
def test_single_list_of_lists(param):
    ...


@pytest.mark.parametrize("a", [1, 2])
@pytest.mark.parametrize(("b", "c"), ((3, 4), (5, 6)))
def test_multiple_decorators(a, b, c):
    pass



import pytest


def test_ok():
    assert [0]


def test_error():
    assert None
    assert False
    assert 0
    assert 0.0
    assert ""
    assert f""
    assert []
    assert ()
    assert {}
    assert list()
    assert set()
    assert tuple()
    assert dict()
    assert frozenset()
    assert list([])
    assert set(set())
    assert tuple("")



from typing import TYPE_CHECKING, List

if TYPE_CHECKING:
    pass  # TCH005


if False:
    pass  # TCH005

if 0:
    pass  # TCH005


def example():
    if TYPE_CHECKING:
        pass  # TCH005
    return


class Test:
    if TYPE_CHECKING:
        pass  # TCH005
    x = 2


if TYPE_CHECKING:
    if 2:
        pass


if TYPE_CHECKING:
    x: List


if False:
    x: List

if 0:
    x: List



# UP035
from collections import Mapping

from collections import Mapping as MAP

from collections import Mapping, Sequence

from collections import Counter, Mapping

from collections import (Counter, Mapping)

from collections import (Counter,
                         Mapping)

from collections import Counter, \
                         Mapping

from collections import Counter, Mapping, Sequence

from collections import Mapping as mapping, Counter

if True:
    from collections import Mapping, Counter

if True:
    if True:
        pass
    from collections import Mapping, Counter

if True: from collections import Mapping

import os
from collections import Counter, Mapping
import sys

if True:
    from collections import (
        Mapping,
        Callable,
        Bad,
        Good,
    )

from typing import Callable, Match, Pattern, List, OrderedDict, AbstractSet, ContextManager

if True: from collections import (
    Mapping, Counter)

# OK
from a import b



# Do this (new version)
from numpy.random import default_rng
rng = default_rng()
vals = rng.standard_normal(10)
more_vals = rng.standard_normal(10)
numbers = rng.integers(high, size=5)

# instead of this (legacy version)
from numpy import random
vals = random.standard_normal(10)
more_vals = random.standard_normal(10)
numbers = random.integers(high, size=5)

import numpy
numpy.random.seed()
numpy.random.get_state()
numpy.random.set_state()
numpy.random.rand()
numpy.random.randn()
numpy.random.randint()
numpy.random.random_integers()
numpy.random.random_sample()
numpy.random.choice()
numpy.random.bytes()
numpy.random.shuffle()
numpy.random.permutation()
numpy.random.beta()
numpy.random.binomial()
numpy.random.chisquare()
numpy.random.dirichlet()
numpy.random.exponential()
numpy.random.f()
numpy.random.gamma()
numpy.random.geometric()
numpy.random.get_state()
numpy.random.gumbel()
numpy.random.hypergeometric()
numpy.random.laplace()
numpy.random.logistic()
numpy.random.lognormal()
numpy.random.logseries()
numpy.random.multinomial()
numpy.random.multivariate_normal()
numpy.random.negative_binomial()
numpy.random.noncentral_chisquare()
numpy.random.noncentral_f()
numpy.random.normal()
numpy.random.pareto()
numpy.random.poisson()
numpy.random.power()
numpy.random.rayleigh()
numpy.random.standard_cauchy()
numpy.random.standard_exponential()
numpy.random.standard_gamma()
numpy.random.standard_normal()
numpy.random.standard_t()
numpy.random.triangular()
numpy.random.uniform()
numpy.random.vonmises()
numpy.random.wald()
numpy.random.weibull()
numpy.random.zipf()



import pytest


def test_ok():
    with pytest.raises(AttributeError):
        [].size


async def test_ok_trivial_with():
    with pytest.raises(AttributeError):
        with context_manager_under_test():
            pass

    with pytest.raises(AttributeError):
        async with context_manager_under_test():
            pass


def test_ok_complex_single_call():
    with pytest.raises(AttributeError):
        my_func(
            [].size,
            [].size,
        )


def test_error_multiple_statements():
    with pytest.raises(AttributeError):
        len([])
        [].size


async def test_error_complex_statement():
    with pytest.raises(AttributeError):
        if True:
            [].size

    with pytest.raises(AttributeError):
        for i in []:
            [].size

    with pytest.raises(AttributeError):
        async for i in []:
            [].size

    with pytest.raises(AttributeError):
        while True:
            [].size

    with pytest.raises(AttributeError):
        with context_manager_under_test():
            [].size

    with pytest.raises(AttributeError):
        async with context_manager_under_test():
            [].size


def test_error_try():
    with pytest.raises(AttributeError):
        try:
            [].size
        except:
            raise



"""This module docstring does not need to be written in imperative mood."""

from functools import cached_property

from gi.repository import GObject

# Bad examples

def bad_liouiwnlkjl():
    """Returns foo."""


def bad_sdgfsdg23245():
    """Constructor for a foo."""


def bad_sdgfsdg23245777():
    """

    Constructor for a boa.

    """


def bad_run_something():
    """Runs something"""

    def bad_nested():
        """Runs other things, nested"""

    bad_nested()


def multi_line():
    """Writes a logical line that
    extends to two physical lines.
    """


# Good examples

def good_run_something():
    """Run away."""

    def good_nested():
        """Run to the hills."""

    good_nested()


def good_construct():
    """Construct a beautiful house."""


def good_multi_line():
    """Write a logical line that
    extends to two physical lines.
    """


good_top_level_var = False
"""This top level assignment attribute docstring does not need to be written in imperative mood."""


# Classes and their members

class Thingy:
    """This class docstring does not need to be written in imperative mood."""

    _beep = "boop"
    """This class attribute docstring does not need to be written in imperative mood."""

    def bad_method(self):
        """This method docstring should be written in imperative mood."""

    @property
    def good_property(self):
        """This property method docstring does not need to be written in imperative mood."""
        return self._beep

    @GObject.Property
    def good_custom_property(self):
        """This property method docstring does not need to be written in imperative mood."""
        return self._beep

    @cached_property
    def good_cached_property(self):
        """This property method docstring does not need to be written in imperative mood."""
        return 42 * 42

    class NestedThingy:
        """This nested class docstring does not need to be written in imperative mood."""


# Test functions

def test_something():
    """This test function does not need to be written in imperative mood.

    pydocstyle's rationale:
    We exclude tests from the imperative mood check, because to phrase
    their docstring in the imperative mood, they would have to start with
    a highly redundant "Test that ..."
    """


def runTest():
    """This test function does not need to be written in imperative mood, either."""



"""
Should emit:
B023 - on lines 12, 13, 16, 28, 29, 30, 31, 40, 42, 50, 51, 52, 53, 61, 68.
"""

functions = []
z = 0

for x in range(3):
    y = x + 1
    # Subject to late-binding problems
    functions.append(lambda: x)
    functions.append(lambda: y)  # not just the loop var

    def f_bad_1():
        return x

    # Actually OK
    functions.append(lambda x: x * 2)
    functions.append(lambda x=x: x)
    functions.append(lambda: z)  # OK because not assigned in the loop

    def f_ok_1(x):
        return x * 2


def check_inside_functions_too():
    ls = [lambda: x for x in range(2)]  # error
    st = {lambda: x for x in range(2)}  # error
    gn = (lambda: x for x in range(2))  # error
    dt = {x: lambda: x for x in range(2)}  # error


async def pointless_async_iterable():
    yield 1


async def container_for_problems():
    async for x in pointless_async_iterable():
        functions.append(lambda: x)  # error

    [lambda: x async for x in pointless_async_iterable()]  # error


a = 10
b = 0
while True:
    a = a_ = a - 1
    b += 1
    functions.append(lambda: a)  # error
    functions.append(lambda: a_)  # error
    functions.append(lambda: b)  # error
    functions.append(lambda: c)  # error, but not a name error due to late binding
    c: bool = a > 3
    if not c:
        break

# Nested loops should not duplicate reports
for j in range(2):
    for k in range(3):
        lambda: j * k  # error


for j, k, l in [(1, 2, 3)]:

    def f():
        j = None  # OK because it's an assignment
        [l for k in range(2)]  # error for l, not for k

        assert a and functions

    a.attribute = 1  # modifying an attribute doesn't make it a loop variable
    functions[0] = lambda: None  # same for an element

for var in range(2):

    def explicit_capture(captured=var):
        return captured


for i in range(3):
    lambda: f"{i}"


# `query` is defined in the function, so also defining it in the loop should be OK.
for name in ["a", "b"]:
    query = name

    def myfunc(x):
        query = x
        query_post = x
        _ = query
        _ = query_post

    query_post = name  # in case iteration order matters


# Bug here because two dict comprehensions reference `name`, one of which is inside
# the lambda.  This should be totally fine, of course.
_ = {
    k: v
    for k, v in reduce(
        lambda data, event: merge_mappings(
            [data, {name: f(caches, data, event) for name, f in xx}]
        ),
        events,
        {name: getattr(group, name) for name in yy},
    ).items()
    if k in backfill_fields
}


# OK to define lambdas if they're immediately consumed, typically as the `key=`
# argument or in a consumed `filter()` (even if a comprehension is better style)
for x in range(2):
    # It's not a complete get-out-of-linting-free construct - these should fail:
    min([None, lambda: x], key=repr)
    sorted([None, lambda: x], key=repr)
    any(filter(bool, [None, lambda: x]))
    list(filter(bool, [None, lambda: x]))
    all(reduce(bool, [None, lambda: x]))

    # But all these should be OK:
    min(range(3), key=lambda y: x * y)
    max(range(3), key=lambda y: x * y)
    sorted(range(3), key=lambda y: x * y)

    any(map(lambda y: x < y, range(3)))
    all(map(lambda y: x < y, range(3)))
    set(map(lambda y: x < y, range(3)))
    list(map(lambda y: x < y, range(3)))
    tuple(map(lambda y: x < y, range(3)))
    sorted(map(lambda y: x < y, range(3)))
    frozenset(map(lambda y: x < y, range(3)))

    any(filter(lambda y: x < y, range(3)))
    all(filter(lambda y: x < y, range(3)))
    set(filter(lambda y: x < y, range(3)))
    list(filter(lambda y: x < y, range(3)))
    tuple(filter(lambda y: x < y, range(3)))
    sorted(filter(lambda y: x < y, range(3)))
    frozenset(filter(lambda y: x < y, range(3)))

    any(reduce(lambda y: x | y, range(3)))
    all(reduce(lambda y: x | y, range(3)))
    set(reduce(lambda y: x | y, range(3)))
    list(reduce(lambda y: x | y, range(3)))
    tuple(reduce(lambda y: x | y, range(3)))
    sorted(reduce(lambda y: x | y, range(3)))
    frozenset(reduce(lambda y: x | y, range(3)))

    import functools

    any(functools.reduce(lambda y: x | y, range(3)))
    all(functools.reduce(lambda y: x | y, range(3)))
    set(functools.reduce(lambda y: x | y, range(3)))
    list(functools.reduce(lambda y: x | y, range(3)))
    tuple(functools.reduce(lambda y: x | y, range(3)))
    sorted(functools.reduce(lambda y: x | y, range(3)))
    frozenset(functools.reduce(lambda y: x | y, range(3)))

# OK because the lambda which references a loop variable is defined in a `return`
# statement, and after we return the loop variable can't be redefined.
# In principle we could do something fancy with `break`, but it's not worth it.
def iter_f(names):
    for name in names:
        if exists(name):
            return lambda: name if exists(name) else None

        if foo(name):
            return [lambda: name]  # known false alarm

        if False:
            return [lambda: i for i in range(3)]  # error


for val in range(3):
    def make_func(val=val):
        def tmp():
            return print(val)

        return tmp


    funcs.append(make_func())



class Fun:
    words = ("how", "fun!")

    def yay(self):
        return self.words

yay = Fun().yay

foo = [4, 5, 6]
bar = [1, 2, 3] + foo
zoob = tuple(bar)
quux = (7, 8, 9) + zoob
spam = quux + (10, 11, 12)
spom = list(spam)
eggs = spom + [13, 14, 15]
elatement = ("we all say", ) + yay()
excitement = ("we all think", ) + Fun().yay()
astonishment = ("we all feel", ) + Fun.words

chain = ['a', 'b', 'c'] + eggs + list(('yes', 'no', 'pants') + zoob)

baz = () + zoob

first = [
    # The order
    1,  # here
    2,  # is
    # extremely
    3,  # critical
    # to preserve
]
second = first + [
    # please
    4,
    # don't
    5,
    # touch
    6,
]

[] + foo + [
]

[] + foo + [  # This will be preserved, but doesn't prevent the fix
]



try:
    pass
except Exception:
    continue

try:
    pass
except:
    continue

try:
    pass
except (Exception,):
    continue

try:
    pass
except (Exception, ValueError):
    continue

try:
    pass
except ValueError:
    continue

try:
    pass
except (ValueError,):
    continue



"""Check that magic values are not used in comparisons."""

user_input = 10

if 10 > user_input:  # [magic-value-comparison]
    pass

if 10 == 100:  # [comparison-of-constants] R0133
    pass

if 1 == 3:  # [comparison-of-constants] R0133
    pass

if 1 == -3:  # [comparison-of-constants] R0133
    pass

x = 0
if 4 == 3 == x:  # [comparison-of-constants] R0133
    pass

time_delta = 7224
ONE_HOUR = 3600

if time_delta > ONE_HOUR:  # correct
    pass

argc = 1

if argc != -1:  # correct
    pass

if argc != 0:  # correct
    pass

if argc != 1:  # correct
    pass

if argc != 2:  # [magic-value-comparison]
    pass

if argc != -2:  # [magic-value-comparison]
    pass

if argc != +2:  # [magic-value-comparison]
    pass

if __name__ == "__main__":  # correct
    pass

ADMIN_PASSWORD = "SUPERSECRET"
input_password = "password"

if input_password == "":  # correct
    pass

if input_password == ADMIN_PASSWORD:  # correct
    pass

if input_password == "Hunter2":  # correct
    pass

PI = 3.141592653589793238
pi_estimation = 3.14

if pi_estimation == 3.141592653589793238:  # [magic-value-comparison]
    pass

if pi_estimation == PI:  # correct
    pass

HELLO_WORLD = b"Hello, World!"
user_input = b"Hello, There!"

if user_input == b"something":  # correct
    pass

if user_input == HELLO_WORLD:  # correct
    pass



from os import chmod, mkdir, makedirs, rename, replace, rmdir
from os import remove, unlink, getcwd, readlink, stat
from os.path import abspath, exists, expanduser, isdir, isfile, islink
from os.path import isabs, join, basename, dirname, samefile, splitext

p = "/foo"

a = abspath(p)
aa = chmod(p)
aaa = mkdir(p)
makedirs(p)
rename(p)
replace(p)
rmdir(p)
remove(p)
unlink(p)
getcwd(p)
b = exists(p)
bb = expanduser(p)
bbb = isdir(p)
bbbb = isfile(p)
bbbbb = islink(p)
readlink(p)
stat(p)
isabs(p)
join(p)
basename(p)
dirname(p)
samefile(p)
splitext(p)
with open(p) as fp:
    fp.read()
open(p).close()



from django.db.models import Model as DjangoModel
from django.db import models
from django.db.models import CharField as SmthCharField


class IncorrectModel(models.Model):
    charfield = models.CharField(max_length=255, null=True)
    textfield = models.TextField(max_length=255, null=True)
    slugfield = models.SlugField(max_length=255, null=True)
    emailfield = models.EmailField(max_length=255, null=True)
    filepathfield = models.FilePathField(max_length=255, null=True)
    urlfield = models.URLField(max_length=255, null=True)


class IncorrectModelWithAlias(DjangoModel):
    charfield = DjangoModel.CharField(max_length=255, null=True)
    textfield = SmthCharField(max_length=255, null=True)
    slugfield = models.SlugField(max_length=255, null=True)
    emailfield = models.EmailField(max_length=255, null=True)
    filepathfield = models.FilePathField(max_length=255, null=True)
    urlfield = models.URLField(max_length=255, null=True)


class IncorrectModelWithoutSuperclass:
    charfield = DjangoModel.CharField(max_length=255, null=True)
    textfield = SmthCharField(max_length=255, null=True)
    slugfield = models.SlugField(max_length=255, null=True)
    emailfield = models.EmailField(max_length=255, null=True)
    filepathfield = models.FilePathField(max_length=255, null=True)
    urlfield = models.URLField(max_length=255, null=True)


class CorrectModel(models.Model):
    charfield = models.CharField(max_length=255, null=False, blank=True)
    textfield = models.TextField(max_length=255, null=False, blank=True)
    slugfield = models.SlugField(max_length=255, null=False, blank=True)
    emailfield = models.EmailField(max_length=255, null=False, blank=True)
    filepathfield = models.FilePathField(max_length=255, null=False, blank=True)
    urlfield = models.URLField(max_length=255, null=False, blank=True)

    charfieldu = models.CharField(max_length=255, null=True, blank=True, unique=True)
    textfieldu = models.TextField(max_length=255, null=True, blank=True, unique=True)
    slugfieldu = models.SlugField(max_length=255, null=True, blank=True, unique=True)
    emailfieldu = models.EmailField(max_length=255, null=True, blank=True, unique=True)
    filepathfieldu = models.FilePathField(
        max_length=255, null=True, blank=True, unique=True
    )
    urlfieldu = models.URLField(max_length=255, null=True, blank=True, unique=True)



class BazMeta(type):
    _private_count = 1

    def __new__(mcs, name, bases, attrs):
        if mcs._private_count <= 5:
            mcs.some_method()

        return super().__new__(mcs, name, bases, attrs)

    def some_method():
        pass


class Bar:
    _private = True

    @classmethod
    def is_private(cls):
        return cls._private


class Foo(metaclass=BazMeta):

    def __init__(self):
        self.public_thing = "foo"
        self._private_thing = "bar"
        self.__really_private_thing = "baz"
        self.bar = Bar()

    def __str__(self):
        return "foo"

    def get_bar():
        if self.bar._private:  # SLF001
            return None
        if self.bar()._private:  # SLF001
            return None
        if Bar._private_thing:  # SLF001
            return None
        if Foo._private_thing:
            return None
        Foo = Bar()
        if Foo._private_thing:  # SLF001
            return None
        return self.bar

    def public_func(self):
        super().public_func()

    def _private_func(self):
        super()._private_func()

    def __really_private_func(self, arg):
        super().__really_private_func(arg)


foo = Foo()

print(foo._private_thing)  # SLF001
print(foo.__really_private_thing)  # SLF001
print(foo._private_func())  # SLF001
print(foo.__really_private_func(1))  # SLF001
print(foo.bar._private)  # SLF001
print(foo()._private_thing)  # SLF001
print(foo()._private_thing__)  # SLF001

print(foo.public_thing)
print(foo.public_func())
print(foo.__dict__)
print(foo.__str__())
print(foo().__class__)
print(foo._asdict())



from __future__ import annotations

from dataclasses import dataclass

import attrs
import pandas
import pyproj
from attrs import frozen

import numpy  # TCH002


@attrs.define(auto_attribs=True)
class A:
    x: pyproj.Transformer


@attrs.define
class B:
    x: pandas.DataFrame


@frozen
class C:
    x: pandas.DataFrame


@dataclass
class D:
    x: numpy.ndarray



try:
    1 / 0
except ValueError as e:
    pass


try:
    1 / 0
except ValueError as e:
    print(e)


def f():
    x = 1
    y = 2
    z = x + y


def f():
    foo = (1, 2)
    (a, b) = (1, 2)

    bar = (1, 2)
    (c, d) = bar

    (x, y) = baz = bar


def f():
    locals()
    x = 1


def f():
    _ = 1
    __ = 1
    _discarded = 1


a = 1


def f():
    global a

    # Used in `c` via `nonlocal`.
    b = 1

    def c():
        # F841
        b = 1

    def d():
        nonlocal b


def f():
    annotations = []
    assert len([annotations for annotations in annotations])


def f():
    def connect():
        return None, None

    with connect() as (connection, cursor):
        cursor.execute("SELECT * FROM users")


def f():
    def connect():
        return None, None

    with connect() as (connection, cursor):
        cursor.execute("SELECT * FROM users")


def f():
    with open("file") as my_file, open("") as ((this, that)):
        print("hello")


def f():
    with (
        open("file") as my_file,
        open("") as ((this, that)),
    ):
        print("hello")


def f():
    exponential, base_multiplier = 1, 2
    hash_map = {
        (exponential := (exponential * base_multiplier) % 3): i + 1 for i in range(2)
    }
    return hash_map


def f(x: int):
    msg1 = "Hello, world!"
    msg2 = "Hello, world!"
    msg3 = "Hello, world!"
    match x:
        case 1:
            print(msg1)
        case 2:
            print(msg2)


def f(x: int):
    import enum

    Foo = enum.Enum("Foo", "A B")
    Bar = enum.Enum("Bar", "A B")
    Baz = enum.Enum("Baz", "A B")

    match x:
        case (Foo.A):
            print("A")
        case [Bar.A, *_]:
            print("A")
        case y:
            pass


def f():
    if any((key := (value := x)) for x in ["ok"]):
        print(key)



import pytest


@pytest.fixture()
def ok_no_scope():
    ...


@pytest.fixture(scope="module")
def ok_other_scope():
    ...


@pytest.fixture(scope="function")
def error():
    ...


@pytest.fixture(scope="function", name="my_fixture")
def error_multiple_args():
    ...


@pytest.fixture(name="my_fixture", scope="function")
def error_multiple_args():
    ...


@pytest.fixture(name="my_fixture", scope="function", **kwargs)
def error_second_arg():
    ...


# pytest.fixture does not take positional arguments, however this 
# tests the general case as we use a helper function that should
# work for all cases.
@pytest.fixture("my_fixture", scope="function")
def error_arg():
    ...


@pytest.fixture(
    scope="function",
    name="my_fixture",
)
def error_multiple_args():
    ...


@pytest.fixture(
    name="my_fixture",
    scope="function",
)
def error_multiple_args():
    ...


@pytest.fixture(
    "hello", 
    name, 
    *args
    , 

    # another comment ,)
    
    scope=\
        "function"  # some comment ),
    ,
    
    name2=name, name3="my_fixture", **kwargs
)
def error_multiple_args():
    ...



if a and False:  # SIM223
    pass

if (a or b) and False:  # SIM223
    pass

if a or (b and False):  # SIM223
    pass

if a or False:
    pass

if False:
    pass

if a and f() and b and g() and False:  # OK
    pass

if a and f() and False and g() and b:  # SIM223
    pass

if False and f() and a and g() and b:  # SIM223
    pass

if a and False and f() and b and g():  # SIM223
    pass


if a or f() or b or g() or True:  # OK
    pass

if a or f() or True or g() or b:  # OK
    pass

if True or f() or a or g() or b:  # OK
    pass

if a or True or f() or b or g():  # OK
    pass


a and "" and False  # SIM223

a and "foo" and False and "bar"  # SIM223

a and 0 and False  # SIM223

a and 1 and False and 2  # SIM223

a and 0.0 and False  # SIM223

a and 0.1 and False and 0.2  # SIM223

a and [] and False  # SIM223

a and list([]) and False  # SIM223

a and [1] and False and [2]  # SIM223

a and list([1]) and False and list([2])  # SIM223

a and {} and False  # SIM223

a and dict() and False  # SIM223

a and {1: 1} and False and {2: 2}  # SIM223

a and dict({1: 1}) and False and dict({2: 2})  # SIM223

a and set() and False  # SIM223

a and set(set()) and False  # SIM223

a and {1} and False and {2}  # SIM223

a and set({1}) and False and set({2})  # SIM223

a and () and False  # SIM222

a and tuple(()) and False  # SIM222

a and (1,) and False and (2,)  # SIM222

a and tuple((1,)) and False and tuple((2,))  # SIM222

a and frozenset() and False  # SIM222

a and frozenset(frozenset()) and False  # SIM222

a and frozenset({1}) and False and frozenset({2})  # SIM222

a and frozenset(frozenset({1})) and False and frozenset(frozenset({2}))  # SIM222


# Inside test `a` is simplified.

bool(a and [] and False and [])  # SIM223

assert a and [] and False and []  # SIM223

if (a and [] and False and []) or (a and [] and False and []):  # SIM223
    pass

0 if a and [] and False and [] else 1  # SIM222

while a and [] and False and []:  # SIM223
    pass

[
    0
    for a in range(10)
    for b in range(10)
    if a and [] and False and []  # SIM223
    if b and [] and False and []  # SIM223
]

{
    0
    for a in range(10)
    for b in range(10)
    if a and [] and False and []  # SIM223
    if b and [] and False and []  # SIM223
}

{
    0: 0
    for a in range(10)
    for b in range(10)
    if a and [] and False and []  # SIM223
    if b and [] and False and []  # SIM223
}

(
    0
    for a in range(10)
    for b in range(10)
    if a and [] and False and []  # SIM223
    if b and [] and False and []  # SIM223
)

# Outside test `a` is not simplified.

a and [] and False and []  # SIM223

if (a and [] and False and []) == (a and []):  # SIM223
    pass

if f(a and [] and False and []):  # SIM223
    pass



class TestClass:
    def __bool__(self):
        ...

    def __bool__(self, x):  # too many mandatory args
        ...

    def __bool__(self, x=1):  # additional optional args OK
        ...

    def __bool__():  # ignored; should be caughty by E0211/N805
        ...

    @staticmethod
    def __bool__():
        ...

    @staticmethod
    def __bool__(x):  # too many mandatory args
        ...

    @staticmethod
    def __bool__(x=1):  # additional optional args OK
        ...

    def __eq__(self, other):  # multiple args
        ...

    def __eq__(self, other=1):  # expected arg is optional
        ...

    def __eq__(self):  # too few mandatory args
        ...

    def __eq__(self, other, other_other):  # too many mandatory args
        ...

    def __round__(self):  # allow zero additional args
        ...

    def __round__(self, x):  # allow one additional args
        ...

    def __round__(self, x, y):  # disallow 2 args
        ...

    def __round__(self, x, y, z=2):  # disallow 3 args even when one is optional
        ...

    def __eq__(self, *args):  # ignore *args
        ...

    def __eq__(self, x, *args):  # extra *args is ok
        ...

    def __eq__(self, x, y, *args):  # too many args with *args
        ...

    def __round__(self, *args):  # allow zero additional args
        ...

    def __round__(self, x, *args):  # allow one additional args
        ...

    def __round__(self, x, y, *args):  # disallow 2 args
        ...

    def __eq__(self, **kwargs):  # ignore **kwargs
        ...

    def __eq__(self, /, other=42):  # ignore positional-only args
        ...

    def __eq__(self, *, other=42):  # ignore positional-only args
        ...



from sklearn.svm import func, SVC, CONST, Klass
from subprocess import N_CLASS, PIPE, Popen, STDOUT
from module import CLASS, Class, CONSTANT, function, BASIC, Apple
from torch.nn import SELU, AClass, A_CONSTANT



import os

# Check all shell functions.
os.execl("true")
os.execle("true")
os.execlp("true")
os.execlpe("true")
os.execv("true")
os.execve("true")
os.execvp("true")
os.execvpe("true")
os.spawnl("true")
os.spawnle("true")
os.spawnlp("true")
os.spawnlpe("true")
os.spawnv("true")
os.spawnve("true")
os.spawnvp("true")
os.spawnvpe("true")
os.startfile("true")



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import AsyncIterator, List


class Example:
    async def example(self) -> AsyncIterator[List[str]]:
        yield 0



def x(y):
    if not y:
        return
    return None  # error


class BaseCache:
    def get(self, key: str) -> str | None:
        print(f"{key} not found")
        return None

    def get(self, key: str) -> None:
        print(f"{key} not found")
        return None



from builtins import *
from builtins import ascii, bytes, compile
from builtins import str as _str
from six.moves import filter, zip, zip_longest
from io import open
import io
import six
import six.moves
import builtins



from a import a1  # import_from
from c import * # import_from_star
import a  # import
import c.d
from z import z1
import b as b1  # import_as
import z

from ..parent import *
from .my import fn
from . import my
from .my.nested import fn2
from ...grandparent import fn3



any([x.id for x in bar])
all([x.id for x in bar])
any(  # first comment
    [x.id for x in bar],  # second comment
)  # third comment
all(  # first comment
    [x.id for x in bar],  # second comment
)  # third comment
any({x.id for x in bar})

# OK
all(x.id for x in bar)
all(x.id for x in bar)
any(x.id for x in bar)
all((x.id for x in bar))


async def f() -> bool:
    return all([await use_greeting(greeting) for greeting in await greetings()])


# Special comment handling
any(
    [  # lbracket comment
        # second line comment
        i.bit_count()
        # random middle comment
        for i in range(5)  # rbracket comment
    ]  # rpar comment
    # trailing comment
)

# Weird case where the function call, opening bracket, and comment are all
# on the same line.
any([  # lbracket comment
        # second line comment
        i.bit_count() for i in range(5)  # rbracket comment
    ]  # rpar comment
)



"""Test: no-blank-line-after-function special-cases around comment handling."""

# OK
def outer():
    """This is a docstring."""
    def inner():
        return

    return inner


# OK
def outer():
    """This is a docstring."""

    def inner():
        return

    return inner


# OK
def outer():
    """This is a docstring."""
    # This is a comment.
    def inner():
        return

    return inner


# OK
def outer():
    """This is a docstring."""

    # This is a comment.
    def inner():
        return

    return inner


# OK
def outer():
    """This is a docstring."""

    # This is a comment.
    # Followed by another comment.
    def inner():
        return

    return inner


# D202
def outer():
    """This is a docstring."""


    def inner():
        return

    return inner


# D202
def outer():
    """This is a docstring."""


    # This is a comment.
    def inner():
        return

    return inner


# D202
def outer():
    """This is a docstring."""

    # This is a comment.

    def inner():
        return

    return inner



@dataclass
class Foo:
    foo: List[str] = field(default_factory=lambda: [])  # PIE807


class FooTable(BaseTable):
    bar = fields.ListField(default=lambda: [])  # PIE807


class FooTable(BaseTable):
    bar = fields.ListField(lambda: [])  # PIE807


@dataclass
class Foo:
    foo: List[str] = field(default_factory=list)


class FooTable(BaseTable):
    bar = fields.ListField(list)


lambda *args, **kwargs: []

lambda *args: []

lambda **kwargs: []



# Adapted from:
#   https://github.com/PyCQA/pylint/blob/b70d2abd7fabe9bfd735a30b593b9cd5eaa36194/tests/functional/g/globals.py

CONSTANT = 1


def FUNC():
    pass


class CLASS:
    pass


def fix_constant(value):
    """All this is ok, but try not to use `global` ;)"""
    global CONSTANT  # [global-statement]
    print(CONSTANT)
    CONSTANT = value


def global_with_import():
    """Should only warn for global-statement when using `Import` node"""
    global sys  # [global-statement]
    import sys


def global_with_import_from():
    """Should only warn for global-statement when using `ImportFrom` node"""
    global namedtuple  # [global-statement]
    from collections import namedtuple


def global_del():
    """Deleting the global name prevents `global-variable-not-assigned`"""
    global CONSTANT  # [global-statement]
    print(CONSTANT)
    del CONSTANT


def global_operator_assign():
    """Operator assigns should only throw a global statement error"""
    global CONSTANT  # [global-statement]
    print(CONSTANT)
    CONSTANT += 1


def global_function_assign():
    """Function assigns should only throw a global statement error"""
    global CONSTANT  # [global-statement]

    def CONSTANT():
        pass

    CONSTANT()


def override_func():
    """Overriding a function should only throw a global statement error"""
    global FUNC  # [global-statement]

    def FUNC():
        pass

    FUNC()


def override_class():
    """Overriding a class should only throw a global statement error"""
    global CLASS  # [global-statement]

    class CLASS:
        pass

    CLASS()



"""Test case: strings used within calls within type annotations."""

from typing import Callable

import bpy
from mypy_extensions import VarArg

from foo import Bar


class LightShow(bpy.types.Operator):
    label = "Create Character"
    name = "lightshow.letter_creation"

    filepath: bpy.props.StringProperty(subtype="FILE_PATH")  # OK


def f(x: Callable[[VarArg("os")], None]):  # F821
    pass


f(Callable[["Bar"], None])
f(Callable[["Baz"], None])



if not (not a):  # SIM208
    pass

if not (not (a == b)):  # SIM208
    pass

if not a:  # OK
    pass

if not a == b:  # OK
    pass

if not a != b:  # OK
    pass

a = not not b  # SIM208

f(not not a)  # SIM208

if 1 + (not (not a)):  # SIM208
    pass



#!/usr/bin/env python3
"""Here's a top-level docstring that's over the limit."""


def f1():
    """Here's a docstring that's also over the limit."""

    x = 1  # Here's a comment that's over the limit, but it's not standalone.

    # Here's a standalone comment that's over the limit.

    x = 2
    # Another standalone that is preceded by a newline and indent toke and is over the limit.

    print("Here's a string that's over the limit, but it's not a docstring.")


"This is also considered a docstring, and is over the limit."


def f2():
    """Here's a multi-line docstring.

    It's over the limit on this line, which isn't the first line in the docstring.
    """


def f3():
    """Here's a multi-line docstring.

    It's over the limit on this line, which isn't the first line in the docstring."""



# OK

import pytest
import pytest as pytest
from notpytest import fixture
from . import fixture
from .pytest import fixture

# Error

import pytest as other_name
from pytest import fixture
from pytest import fixture as other_name



from typing import ParamSpec, TypeVar, TypeVarTuple

T = TypeVar("T")  # Error: TypeVars in stubs must start with _

TTuple = TypeVarTuple("TTuple")  # Error: TypeVarTuples must also start with _

P = ParamSpec("P")  # Error: ParamSpecs must start with _

_T = TypeVar("_T")  # OK

_TTuple = TypeVarTuple("_TTuple")  # OK

_P = ParamSpec("_P")  # OK

def f():
    T = TypeVar("T")  # OK



# Errors
assert my_mock.not_called()
assert my_mock.called_once_with()
assert my_mock.not_called
assert my_mock.called_once_with
my_mock.assert_not_called
my_mock.assert_called
my_mock.assert_called_once_with
my_mock.assert_called_once_with
MyMock.assert_called_once_with

# OK
assert my_mock.call_count == 1
assert my_mock.called
my_mock.assert_not_called()
my_mock.assert_called()
my_mock.assert_called_once_with()
"""like :meth:`Mock.assert_called_once_with`"""
"""like :meth:`MagicMock.assert_called_once_with`"""



import lib6
import lib2
import lib5
import lib1
import lib3
import lib4

import foo
import z
from foo import bar
from lib1 import foo
from lib2 import foo
from lib1.lib2 import foo
from foo.lib1.bar import baz
from lib4 import lib1
from lib5 import lib2
from lib4 import lib2
from lib5 import lib1

import lib3.lib4
import lib3.lib4.lib5
from lib3.lib4 import foo
from lib3.lib4.lib5 import foo



min(1, 2, 3)
min(1, min(2, 3))
min(1, min(2, min(3, 4)))
min(1, foo("a", "b"), min(3, 4))
min(1, max(2, 3))
max(1, 2, 3)
max(1, max(2, 3))
max(1, max(2, max(3, 4)))
max(1, foo("a", "b"), max(3, 4))

# These should not trigger; we do not flag cases with keyword args.
min(1, min(2, 3), key=test)
min(1, min(2, 3, key=test))
# This will still trigger, to merge the calls without keyword args.
min(1, min(2, 3, key=test), min(4, 5))

# Don't provide a fix if there are comments within the call.
min(
    1,  # This is a comment.
    min(2, 3),
)



import sys
import typing
from typing import Annotated, Literal, TypeAlias, TypeVar

import typing_extensions

def f(x: "int"): ...  # Y020 Quoted annotations should never be used in stubs
def g(x: list["int"]): ...  # Y020 Quoted annotations should never be used in stubs
_T = TypeVar("_T", bound="int")  # Y020 Quoted annotations should never be used in stubs

def h(w: Literal["a", "b"], x: typing.Literal["c"], y: typing_extensions.Literal["d"], z: _T) -> _T: ...

def j() -> "int": ...  # Y020 Quoted annotations should never be used in stubs
Alias: TypeAlias = list["int"]  # Y020 Quoted annotations should never be used in stubs

class Child(list["int"]):  # Y020 Quoted annotations should never be used in stubs
    """Documented and guaranteed useful."""  # Y021 Docstrings should not be included in stubs

if sys.platform == "linux":
    f: "int"  # Y020 Quoted annotations should never be used in stubs
elif sys.platform == "win32":
    f: "str"  # Y020 Quoted annotations should never be used in stubs
else:
    f: "bytes"  # Y020 Quoted annotations should never be used in stubs

# These two shouldn't trigger Y020 -- empty strings can't be "quoted annotations"
k = ""  # Y052 Need type annotation for "k"
el = r""  # Y052 Need type annotation for "el"



from __future__ import annotations

import pathlib
from typing import TYPE_CHECKING

import attrs
from attrs import frozen

import numpy

if TYPE_CHECKING:
    import datetime  # TCH004
    from array import array  # TCH004

    import pandas  # TCH004
    import pyproj


@attrs.define(auto_attribs=True)
class A:
    x: datetime.datetime


@attrs.define
class B:
    x: pandas.DataFrame


@frozen(auto_attribs=True)
class C:
    x: pathlib.Path


@frozen
class D:
    x: array


@dataclass
class E:
    x: pyproj.Transformer


@attrs.define
class F:
    x: numpy.ndarray



import sys, math
from os import path, uname
from json import detect_encoding
from json import dump
from json import dumps as json_dumps
from json import load
from json import loads as json_loads
from logging.handlers import StreamHandler, FileHandler

# comment 1
from third_party import lib1, lib2, \
     lib3, lib7, lib5, lib6
# comment 2
from third_party import lib4

from foo import bar  # comment 3
from foo2 import bar2  # comment 4
from foo3 import bar3, baz3  # comment 5

# comment 6
from bar import (
     a, # comment 7
     b, # comment 8
)

# comment 9
from baz import *  # comment 10



import pytest


def test_xxx():
    pytest.fail("this is a failure")  # Test OK arg


def test_xxx():
    pytest.fail(msg="this is a failure")  # Test OK kwarg


def test_xxx():  # Error
    pytest.fail()
    pytest.fail("")
    pytest.fail(f"")
    pytest.fail(msg="")
    pytest.fail(msg=f"")



import os
import os.path

p = "/foo"

a = os.path.abspath(p)
aa = os.chmod(p)
aaa = os.mkdir(p)
os.makedirs(p)
os.rename(p)
os.replace(p)
os.rmdir(p)
os.remove(p)
os.unlink(p)
os.getcwd(p)
b = os.path.exists(p)
bb = os.path.expanduser(p)
bbb = os.path.isdir(p)
bbbb = os.path.isfile(p)
bbbbb = os.path.islink(p)
os.readlink(p)
os.stat(p)
os.path.isabs(p)
os.path.join(p)
os.path.basename(p)
os.path.dirname(p)
os.path.samefile(p)
os.path.splitext(p)
with open(p) as fp:
    fp.read()
open(p).close()
os.getcwdb(p)



import sys

if sys.platform == "linus": ...  # Error: PYI008 Unrecognized platform `linus`

if sys.platform != "linux": ...  # OK

if sys.platform == "win32": ...  # OK

if sys.platform != "darwin": ...  # OK

if sys.platform == "cygwin": ...  # OK



class CustomException(Exception):
    pass


def func():
    a = 1
    if a == 1:
        raise CustomException("Long message")
    elif a == 2:
        raise CustomException("Short")  # This is acceptable
    elif a == 3:
        raise CustomException("its_code_not_message")  # This is acceptable


def ignore():
    try:
        a = 1
    except Exception as ex:
        # This is another violation, but this specific analyzer shouldn't care
        raise ex


class BadArgCantBeEven(Exception):
    pass


class GoodArgCantBeEven(Exception):
    def __init__(self, arg):
        super().__init__(f"The argument '{arg}' should be even")


def bad(a):
    if a % 2 == 0:
        raise BadArgCantBeEven(f"The argument '{a}' should be even")


def another_bad(a):
    if a % 2 == 0:
        raise BadArgCantBeEven(f"The argument {a} should not be odd.")


def and_another_bad(a):
    if a % 2 == 0:
        raise BadArgCantBeEven("The argument `a` should not be odd.")


def good(a: int):
    if a % 2 == 0:
        raise GoodArgCantBeEven(a)


def another_good(a):
    if a % 2 == 0:
        raise GoodArgCantBeEven(a)



from __future__ import annotations


def f():
    # Even in strict mode, this shouldn't rase an error, since `pkg` is used at runtime,
    # and implicitly imports `pkg.bar`.
    import pkg
    import pkg.bar

    def test(value: pkg.bar.A):
        return pkg.B()


def f():
    # Even in strict mode, this shouldn't rase an error, since `pkg.bar` is used at
    # runtime, and implicitly imports `pkg`.
    import pkg
    import pkg.bar

    def test(value: pkg.A):
        return pkg.bar.B()


def f():
    # In un-strict mode, this shouldn't rase an error, since `pkg` is used at runtime.
    import pkg
    from pkg import A

    def test(value: A):
        return pkg.B()


def f():
    # In un-strict mode, this shouldn't rase an error, since `pkg` is used at runtime.
    from pkg import A, B

    def test(value: A):
        return B()


def f():
    # Even in strict mode, this shouldn't rase an error, since `pkg.baz` is used at
    # runtime, and implicitly imports `pkg.bar`.
    import pkg.bar
    import pkg.baz

    def test(value: pkg.bar.A):
        return pkg.baz.B()


def f():
    # In un-strict mode, this _should_ rase an error, since `pkg` is used at runtime.
    import pkg
    from pkg.bar import A

    def test(value: A):
        return pkg.B()



"""Test: match statements."""
from dataclasses import dataclass


@dataclass
class Car:
    make: str
    model: str


def f():
    match Car("Toyota", "Corolla"):
        case Car("Toyota", model):
            print(model)
        case Car(make, "Corolla"):
            print(make)


def f(provided: int) -> int:
    match provided:
        case True:
            return captured  # F821


def f(provided: int) -> int:
    match provided:
        case captured:
            return captured


def f(provided: int) -> int:
    match provided:
        case [captured, *_]:
            return captured


def f(provided: int) -> int:
    match provided:
        case [*captured]:
            return captured


def f(provided: int) -> int:
    match provided:
        case {**captured}:
            return captured



import unittest


class Test(unittest.TestCase):
    def test_xxx(self):
        assert 1 == 1  # OK no parameters

    def test_assert_true(self):
        expr = 1
        msg = "Must be True"
        self.assertTrue(expr)  # Error
        self.assertTrue(expr=expr)  # Error
        self.assertTrue(expr, msg)  # Error
        self.assertTrue(expr=expr, msg=msg)  # Error
        self.assertTrue(msg=msg, expr=expr)  # Error
        self.assertTrue(*(expr, msg))  # Error, unfixable
        self.assertTrue(**{"expr": expr, "msg": msg})  # Error, unfixable
        self.assertTrue(msg=msg, expr=expr, unexpected_arg=False)  # Error, unfixable
        self.assertTrue(msg=msg)  # Error, unfixable
        (
            self.assertIsNotNone(value)  # Error, unfixable
            if expect_condition
            else self.assertIsNone(value)  # Error, unfixable
        )
        return self.assertEqual(True,  False)  # Error, unfixable

    def test_assert_false(self):
        self.assertFalse(True)  # Error

    def test_assert_equal(self):
        self.assertEqual(1, 2)  # Error

    def test_assert_not_equal(self):
        self.assertNotEqual(1, 1)  # Error

    def test_assert_greater(self):
        self.assertGreater(1, 2)  # Error

    def test_assert_greater_equal(self):
        self.assertGreaterEqual(1, 2)  # Error

    def test_assert_less(self):
        self.assertLess(2, 1)  # Error

    def test_assert_less_equal(self):
        self.assertLessEqual(1, 2)  # Error

    def test_assert_in(self):
        self.assertIn(1, [2, 3])  # Error

    def test_assert_not_in(self):
        self.assertNotIn(2, [2, 3])  # Error

    def test_assert_is_none(self):
        self.assertIsNone(0)  # Error

    def test_assert_is_not_none(self):
        self.assertIsNotNone(0)  # Error

    def test_assert_is(self):
        self.assertIs([], [])  # Error

    def test_assert_is_not(self):
        self.assertIsNot(1, 1)  # Error

    def test_assert_is_instance(self):
        self.assertIsInstance(1, str)  # Error

    def test_assert_is_not_instance(self):
        self.assertNotIsInstance(1, int)  # Error

    def test_assert_regex(self):
        self.assertRegex("abc", r"def")  # Error

    def test_assert_not_regex(self):
        self.assertNotRegex("abc", r"abc")  # Error

    def test_assert_regexp_matches(self):
        self.assertRegexpMatches("abc", r"def")  # Error

    def test_assert_not_regexp_matches(self):
        self.assertNotRegex("abc", r"abc")  # Error



from numpy import (
	cos,
	int8,
	int16,
	int32,
	int64,
	sin,
	tan,
	uint8,
	uint16,
	uint32,
	uint64,
)

if True:
	# inside nested block
	from numpy import (
		cos,
		int8,
		int16,
		int32,
		int64,
		sin,
		tan,
		uint8,
		uint16,
		uint32,
		uint64,
	)



import warnings

"""
Should emit:
B028 - on lines 8 and 9
"""

warnings.warn(DeprecationWarning("test"))
warnings.warn(DeprecationWarning("test"), source=None)
warnings.warn(DeprecationWarning("test"), source=None, stacklevel=2)
warnings.warn(DeprecationWarning("test"), stacklevel=1)



#: E731
f = lambda x: 2 * x
#: E731
f = lambda x: 2 * x
#: E731
while False:
    this = lambda y, z: 2 * x
#: E731
f = lambda: (yield 1)
#: E731
f = lambda: (yield from g())
#: E731
class F:
    f = lambda x: 2 * x


f = object()
f.method = lambda: "Method"
f = {}
f["a"] = lambda x: x**2
f = []
f.append(lambda x: x**2)
f = g = lambda x: x**2
lambda: "no-op"

# Annotated
from typing import Callable, ParamSpec

P = ParamSpec("P")

# ParamSpec cannot be used in this context, so do not preserve the annotation.
f: Callable[P, int] = lambda *args: len(args)
f: Callable[[], None] = lambda: None
f: Callable[..., None] = lambda a, b: None
f: Callable[[int], int] = lambda x: 2 * x

# Let's use the `Callable` type from `collections.abc` instead.
from collections.abc import Callable

f: Callable[[str, int], str] = lambda a, b: a * b
f: Callable[[str, int], tuple[str, int]] = lambda a, b: (a, b)
f: Callable[[str, int, list[str]], list[str]] = lambda a, b, /, c: [*c, a * b]


# Override `Callable`
class Callable:
    pass


# Do not copy the annotation from here on out.
f: Callable[[str, int], str] = lambda a, b: a * b



"""Test case for autofixing F841 violations."""


def f():
    x = 1
    y = 2

    z = 3
    print(z)


def f():
    x: int = 1
    y: int = 2

    z: int = 3
    print(z)


def f():
    with foo() as x1:
        pass

    with foo() as (x2, y2):
        pass

    with (foo() as x3, foo() as y3, foo() as z3):
        pass


def f():
    (x1, y1) = (1, 2)
    (x2, y2) = coords2 = (1, 2)
    coords3 = (x3, y3) = (1, 2)


def f():
    try:
        1 / 0
    except ValueError as x1:
        pass

    try:
        1 / 0
    except (ValueError, ZeroDivisionError) as x2:
        pass


def f(a, b):
    x = (
        a()
        if a is not None
        else b
    )

    y = \
        a() if a is not None else b


def f(a, b):
    x = (
        a
        if a is not None
        else b
    )

    y = \
        a if a is not None else b


def f():
    with Nested(m) as (cm):
        pass


def f():
    with (Nested(m) as (cm),):
        pass


def f():
    with Nested(m) as (x, y):
        pass


def f():
    toplevel = tt = lexer.get_token()
    if not tt:
        break


def f():
    toplevel = tt = lexer.get_token()


def f():
    toplevel = (a, b) = lexer.get_token()


def f():
    (a, b) = toplevel = lexer.get_token()


def f():
    toplevel = tt = 1



x = [1, 2, 3]
list(list(x))
list(tuple(x))
tuple(list(x))
tuple(tuple(x))
set(set(x))
set(list(x))
set(tuple(x))
set(sorted(x))
set(sorted(x, key=lambda y: y))
set(reversed(x))
sorted(list(x))
sorted(tuple(x))
sorted(sorted(x))
sorted(sorted(x, key=lambda y: y))
sorted(reversed(x))
sorted(list(x), key=lambda y: y)
tuple(
    list(
        [x, 3, "hell"\
        "o"]
    )
)



# SIM117
with A() as a:
    with B() as b:
        print("hello")

# SIM117
with A():
    with B():
        with C():
            print("hello")

# SIM117
with A() as a:
    # Unfixable due to placement of this comment.
    with B() as b:
        print("hello")

# SIM117
with A() as a:
    with B() as b:
        # Fixable due to placement of this comment.
        print("hello")

# OK
with A() as a:
    a()
    with B() as b:
        print("hello")

# OK
with A() as a:
    with B() as b:
        print("hello")
    a()

# OK
async with A() as a:
    with B() as b:
        print("hello")

# OK
with A() as a:
    async with B() as b:
        print("hello")

# OK
async with A() as a:
    async with B() as b:
        print("hello")

while True:
    # SIM117
    with A() as a:
        with B() as b:
            """this
is valid"""

            """the indentation on
            this line is significant"""

            "this is" \
"allowed too"

            ("so is"
"this for some reason")

# SIM117
with (
    A() as a,
    B() as b,
):
    with C() as c:
        print("hello")

# SIM117
with A() as a:
    with (
        B() as b,
        C() as c,
    ):
        print("hello")

# SIM117
with (
    A() as a,
    B() as b,
):
    with (
        C() as c,
        D() as d,
    ):
        print("hello")



a = True if b else False  # SIM210

a = True if b != c else False  # SIM210

a = True if b + c else False  # SIM210

a = False if b else True  # OK


def f():
    # OK
    def bool():
        return False

    a = True if b else False



import functools

import pytest


@pytest.fixture()
def my_fixture():  # OK return
    return 0


@pytest.fixture()
def my_fixture():  # OK yield
    resource = acquire_resource()
    yield resource
    resource.release()


@pytest.fixture()
def my_fixture():  # OK other request
    request = get_request()
    request.addfinalizer(finalizer)
    return request


def create_resource(arg, request):  # OK other function
    resource = Resource(arg)
    request.addfinalizer(resource.release)
    return resource


@pytest.fixture()
def resource_factory(request):
    return functools.partial(create_resource, request=request)


@pytest.fixture()
def resource_factory(request):  # OK other function
    def create_resource(arg) -> Resource:
        resource = Resource(arg)
        request.addfinalizer(resource.release)
        return resource

    return create_resource


@pytest.fixture()
def my_fixture(request):  # Error return
    resource = acquire_resource()
    request.addfinalizer(resource.release)
    return resource


@pytest.fixture()
def my_fixture(request):  # Error yield
    resource = acquire_resource()
    request.addfinalizer(resource.release)
    yield resource
    resource  # prevent PT022



"""
Violation:
Use '.exception' over '.error' inside except blocks
"""

import logging

logger = logging.getLogger(__name__)


def bad():
    try:
        a = 1
    except Exception:
        logging.error("Context message here")

        if True:
            logging.error("Context message here")


def bad():
    try:
        a = 1
    except Exception:
        logger.error("Context message here")

        if True:
            logger.error("Context message here")


def bad():
    try:
        a = 1
    except Exception:
        log.error("Context message here")

        if True:
            log.error("Context message here")


def bad():
    try:
        a = 1
    except Exception:
        self.logger.error("Context message here")

        if True:
            self.logger.error("Context message here")


def good():
    try:
        a = 1
    except Exception:
        logger.exception("Context message here")


def good():
    try:
        a = 1
    except Exception:
        foo.exception("Context message here")



import logging

logging.warning("Hello %s", "World!", "again")  # [logging-too-many-args]

logging.warning("Hello %s", "World!", "again", something="else")

logging.warning("Hello %s", "World!")

# do not handle calls with *args
logging.error("Example log %s, %s", "foo", "bar", "baz", *args)

# do not handle calls with **kwargs
logging.error("Example log %s, %s", "foo", "bar", "baz", **kwargs)

# do not handle keyword arguments
logging.error("%(objects)d modifications: %(modifications)d errors: %(errors)d", {"objects": 1, "modifications": 1, "errors": 1})

import warning

warning.warning("Hello %s", "World!", "again")



x = 1  # pyright: ignore
x = 1  # pyright:ignore
x = 1  # pyright: ignore[attr-defined]  # pyright: ignore

x = 1
x = 1  # pyright ignore
x = 1  # pyright ignore  # noqa
x = 1  # pyright: ignore[attr-defined]
x = 1  # pyright: ignore[attr-defined, name-defined]
x = 1  # pyright: ignore[attr-defined]  # pyright: ignore[type-mismatch]
x = 1  # pyright: ignore[type-mismatch]  # noqa
x = 1  # pyright: ignore [attr-defined]
x = 1  # pyright: ignore [attr-defined, name-defined]
x = 1  # pyright: ignore [type-mismatch]  # noqa
x = 1  # pyright: Union[int, str]
x = 1  # pyright: ignoreme



import pytest
from pytest import fixture
from pytest import fixture as aliased


# `import pytest`


@pytest.fixture
def no_parentheses():
    return 42


@pytest.fixture()
def parentheses_no_params():
    return 42


@pytest.fixture(scope="module")
def parentheses_with_params():
    return 42


@pytest.fixture(

)
def parentheses_no_params_multiline():
    return 42


# `from pytest import fixture`


@fixture
def imported_from_no_parentheses():
    return 42


@fixture()
def imported_from_parentheses_no_params():
    return 42


@fixture(scope="module")
def imported_from_parentheses_with_params():
    return 42


@fixture(

)
def imported_from_parentheses_no_params_multiline():
    return 42


# `from pytest import fixture as aliased`


@aliased
def aliased_no_parentheses():
    return 42


@aliased()
def aliased_parentheses_no_params():
    return 42


@aliased(scope="module")
def aliased_parentheses_with_params():
    return 42


@aliased(

)
def aliased_parentheses_no_params_multiline():
    return 42



isinstance(1, (int, float))  # UP038
issubclass("yes", (int, float, str))  # UP038

isinstance(1, int)  # OK
issubclass("yes", int)  # OK
isinstance(1, int | float)  # OK
issubclass("yes", int | str)  # OK
isinstance(1, ())  # OK
isinstance(1, (int, *(str, bytes)))  # OK



x = set(x for x in range(3))
x = set(
    x for x in range(3)
)
y = f'{set(a if a < 6 else 0  for a in range(3))}'
_ = '{}'.format(set(a if a < 6 else 0  for a in range(3)))
print(f'Hello {set(a for a in range(3))} World')

def set(*args, **kwargs):
    return None


set(x for x in range(3))



import pytest


def test_ok():
    assert something
    assert something or something_else
    assert something or something_else and something_third
    assert not (something and something_else)
    assert something, "something message"
    assert something or something_else and something_third, "another message"


def test_error():
    assert something and something_else
    assert something and something_else and something_third
    assert something and not something_else
    assert something and (something_else or something_third)
    assert not something and something_else
    assert not (something or something_else)
    assert not (something or something_else or something_third)
    assert something and something_else == """error
    message
    """

    # recursive case
    assert not (a or not (b or c))
    assert not (a or not (b and c))

    # detected, but no autofix for messages
    assert something and something_else, "error message"
    assert not (something or something_else and something_third), "with message"
    # detected, but no autofix for mixed conditions (e.g. `a or b and c`)
    assert not (something or something_else and something_third)
    # detected, but no autofix for parenthesized conditions
    assert (
        something
        and something_else
        == """error
message
"""
    )



from typing import ParamSpec, TypeVar, TypeVarTuple

T = TypeVar("T")  # OK

TTuple = TypeVarTuple("TTuple")  # OK

P = ParamSpec("P")  # OK

_T = TypeVar("_T")  # OK

_TTuple = TypeVarTuple("_TTuple")  # OK

_P = ParamSpec("_P")  # OK


def f():
    T = TypeVar("T")  # OK



from typing import TypedDict, NotRequired, Literal
import typing

# dict literal
MyType = TypedDict("MyType", {"a": int, "b": str})

# dict call
MyType = TypedDict("MyType", dict(a=int, b=str))

# kwargs
MyType = TypedDict("MyType", a=int, b=str)

# Empty TypedDict
MyType = TypedDict("MyType")

# Literal values
MyType = TypedDict("MyType", {"a": "hello"})
MyType = TypedDict("MyType", a="hello")

# NotRequired
MyType = TypedDict("MyType", {"a": NotRequired[dict]})

# total
MyType = TypedDict("MyType", {"x": int, "y": int}, total=False)

# using Literal type
MyType = TypedDict("MyType", {"key": Literal["value"]})

# using namespace TypedDict
MyType = typing.TypedDict("MyType", {"key": int})

# invalid identifiers (OK)
MyType = TypedDict("MyType", {"in": int, "x-y": int})

# unpacking (OK)
c = {"c": float}
MyType = TypedDict("MyType", {"a": int, "b": str, **c})

# Empty dict literal
MyType = TypedDict("MyType", {})

# Empty dict call
MyType = TypedDict("MyType", dict())



# ==> bad_function_call.py <==
bad_function_call(
    param1='test',
    param2='test'
)
# ==> bad_list.py <==
bad_list = [
    1,
    2,
    3
]

bad_list_with_comment = [
    1,
    2,
    3
    # still needs a comma!
]

bad_list_with_extra_empty = [
    1,
    2,
    3



]

# ==> bare.py <==
bar = 1, 2

foo = 1

foo = (1,)

foo = 1,

bar = 1; foo = bar,

foo = (
3,
4,
)

foo = 3, 

class A(object):
 foo = 3
 bar = 10,
 foo_bar = 2

a = ('a',)

from foo import bar, baz

group_by = function_call('arg'),

group_by = ('foobar' * 3),

def foo():
    return False,

# ==> callable_before_parenth_form.py <==
def foo(
    bar,
):
    pass

{'foo': foo}['foo'](
    bar
)

{'foo': foo}['foo'](
    bar,
)

(foo)(
    bar
)

(foo)[0](
    bar,
)

[foo][0](
    bar
)

[foo][0](
    bar,
)

# ==> comment_good_dict.py <==
multiline_good_dict = {
    "good": 123,  # this is a good number
}

# ==> dict_comprehension.py <==
not_a_dict = {
    x: y
    for x, y in ((1, 2), (3, 4))
}

# ==> good_empty_comma_context.py <==
def func2(
):
    pass


func2(

)

func2(
)

[
]

[

]

(
)

(

)

{

}

# ==> good_list.py <==
stuff = [
   'a',
   'b',
   # more stuff will go here
]

more_stuff = [
    'a',
    'b',



]

# ==> keyword_before_parenth_form/base_bad.py <==
from x import (
    y
)

assert(
    SyntaxWarning,
    ThrownHere,
    Anyway
)

# async await is fine outside an async def
# ruff: RustPython tokenizer treats async/await as keywords, not applicable.

# def await(
#     foo
# ):
#     async(
#         foo
#     )

# def async(
#     foo
# ):
#     await(
#         foo
#     )

# ==> keyword_before_parenth_form/base.py <==
from x import (
    y,
)

assert(
    SyntaxWarning,
    ThrownHere,
    Anyway,
)

assert (
    foo
)

assert (
    foo and
    bar
)

if(
    foo and
    bar
):
    pass
elif(
    foo and
    bar
):
    pass

for x in(
    [1,2,3]
):
    print(x)


(x for x in (
    [1, 2, 3]
))

(
    'foo'
) is (
    'foo'
)

if (
    foo and
    bar
) or not (
    foo
) or (
    spam
):
    pass

def xyz():
    raise(
        Exception()
    )

def abc():
    return(
        3
    )

while(
    False
):
    pass

with(
    loop
):
    pass

def foo():
    yield (
        "foo"
    )

# async await is fine outside an async def
# ruff: RustPython tokenizer treats async/await as keywords, not applicable.

# def await(
#     foo,
# ):
#     async(
#         foo,
#     )

# def async(
#     foo,
# ):
#     await(
#         foo,
#     )

# ==> keyword_before_parenth_form/py3.py <==
# Syntax error in Py2

def foo():
    yield from (
        foo
    )

# ==> list_comprehension.py <==
not_a_list = [
    s.strip()
    for s in 'foo, bar, baz'.split(',')
]

# ==> multiline_bad_dict.py <==
multiline_bad_dict = {
    "bad": 123
}
# ==> multiline_bad_function_def.py <==
def func_good(
        a = 3,
        b = 2):
    pass


def func_bad(
        a = 3,
        b = 2
    ):
    pass

# ==> multiline_bad_function_one_param.py <==
def func(
        a = 3
    ):
    pass


func(
    a = 3
)

# ==> multiline_bad_or_dict.py <==
multiline_bad_or_dict = {
    "good": True or False,
    "bad": 123
}

# ==> multiline_good_dict.py <==
multiline_good_dict = {
    "good": 123,
}
# ==> multiline_good_single_keyed_for_dict.py <==
good_dict = {
    "good": x for x in y
}

# ==> multiline_if.py <==
if (
    foo
    and bar
):
    print("Baz")

# ==> multiline_index_access.py <==
multiline_index_access[
    "good"
]

multiline_index_access_after_function()[
    "good"
]

multiline_index_access_after_inline_index_access['first'][
    "good"
]

multiline_index_access[
    "probably fine",
]

[0, 1, 2][
    "good"
]

[0, 1, 2][
    "probably fine",
]

multiline_index_access[
    "probably fine",
    "not good"
]

multiline_index_access[
    "fine",
    "fine",
    :
    "not good"
]

# ==> multiline_string.py <==
s = (
    'this' +
    'is a string'
)

s2 = (
    'this'
    'is also a string'
)

t = (
    'this' +
    'is a tuple',
)

t2 = (
    'this'
    'is also a tuple',
)

# ==> multiline_subscript_slice.py <==
multiline_index_access[
    "fine",
    "fine"
    :
    "not fine"
]

multiline_index_access[
    "fine"
    "fine"
    :
    "fine"
    :
    "fine"
]

multiline_index_access[
    "fine"
    "fine",
    :
    "fine",
    :
    "fine",
]

multiline_index_access[
    "fine"
    "fine",
    :
    "fine"
    :
    "fine",
    "not fine"
]

multiline_index_access[
    "fine"
    "fine",
    :
    "fine",
    "fine"
    :
    "fine",
]

multiline_index_access[
    lambda fine,
    fine,
    fine: (0,)
    :
    lambda fine,
    fine,
    fine: (0,),
    "fine"
    :
    "fine",
]

# ==> one_line_dict.py <==
one_line_dict = {"good": 123}
# ==> parenth_form.py <==
parenth_form = (
    a +
    b +
    c
)

parenth_form_with_lambda = (
    lambda x, y: 0
)

parenth_form_with_default_lambda = (
    lambda x=(
        lambda
        x,
        y,
        :
        0
    ),
    y = {a: b},
    :
    0
)

# ==> prohibited.py <==
foo = ['a', 'b', 'c',]

bar = { a: b,}

def bah(ham, spam,):
    pass

(0,)

(0, 1,)

foo = ['a', 'b', 'c', ]

bar = { a: b, }

def bah(ham, spam, ):
    pass

(0, )

(0, 1, )

image[:, :, 0]

image[:,]

image[:,:,]

lambda x, :

# ==> unpack.py <==
def function(
    foo,
    bar,
    **kwargs
):
    pass

def function(
    foo,
    bar,
    *args
):
    pass

def function(
    foo,
    bar,
    *args,
    extra_kwarg
    ):
    pass

result = function(
    foo,
    bar,
    **kwargs
)

result = function(
    foo,
    bar,
    **not_called_kwargs
)

def foo(
    ham,
    spam,
    *args,
    kwarg_only
    ):
    pass

# In python 3.5 if it's not a function def, commas are mandatory.

foo(
    **kwargs
)

{
    **kwargs
}

(
    *args
)

{
    *args
}

[
    *args
]

def foo(
    ham,
    spam,
    *args
    ):
    pass

def foo(
    ham,
    spam,
    **kwargs
    ):
    pass

def foo(
    ham,
    spam,
    *args,
    kwarg_only
    ):
    pass

# In python 3.5 if it's not a function def, commas are mandatory.

foo(
    **kwargs,
)

{
    **kwargs,
}

(
    *args,
)

{
    *args,
}

[
    *args,
]

result = function(
    foo,
    bar,
    **{'ham': spam}
)

# Make sure the COM812 and UP034 rules don't autofix simultaneously and cause a syntax error.
the_first_one = next(
    (i for i in range(10) if i // 2 == 0)  # COM812 fix should include the final bracket
)



import pytest


@pytest.fixture()
def ok_no_parameters():
    return 0


@pytest.fixture
def ok_without_parens():
    return 0


@pytest.yield_fixture()
def error_without_parens():
    return 0


@pytest.yield_fixture
def error_with_parens():
    return 0



import sys

if            sys.version_info < (3,0):
    print("py2")
    for item in range(10):
        print(f"PY2-{item}")
else                                :
    print("py3")
    for item in range(10):
        print(f"PY3-{item}")

if False:
    if            sys.version_info < (3,0):
        print("py2")
        for item in range(10):
            print(f"PY2-{item}")
    else                                :
        print("py3")
        for item in range(10):
            print(f"PY3-{item}")


if            sys.version_info < (3,0): print("PY2!")
else                               : print("PY3!")



import math
import os
import sys
from math import inf

import numpy as np

def f12(
    x,
    y: str = os.pathsep,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f11(*, x: str = "x") -> None: ...  # OK
def f13(
    x: list[str] = [  # OK
        "foo",
        "bar",
        "baz",
    ]
) -> None: ...
def f14(
    x: tuple[str, ...] = (  # OK
        "foo",
        "bar",
        "baz",
    )
) -> None: ...
def f15(
    x: set[str] = {  # OK
        "foo",
        "bar",
        "baz",
    }
) -> None: ...
def f151(x: dict[int, int] = {1: 2}) -> None: ...  # Ok
def f152(
    x: dict[
        int, int
    ] = {  # Error PYI011 Only simple default values allowed for typed arguments
        1: 2,
        **{3: 4},
    }
) -> None: ...
def f153(
    x: list[
        int
    ] = [  # Error PYI011 Only simple default values allowed for typed arguments
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        10,
        11,
    ]
) -> None: ...
def f154(
    x: tuple[
        str, tuple[str, ...]
    ] = (  # Error PYI011 Only simple default values allowed for typed arguments
        "foo",
        ("bar", "baz"),
    )
) -> None: ...
def f141(
    x: list[
        int
    ] = [  # Error PYI011 Only simple default values allowed for typed arguments
        *range(10)
    ],
) -> None: ...
def f142(
    x: list[
        int
    ] = list(  # Error PYI011 Only simple default values allowed for typed arguments
        range(10)
    ),
) -> None: ...
def f16(
    x: frozenset[
        bytes
    ] = frozenset(  # Error PYI011 Only simple default values allowed for typed arguments
        {b"foo", b"bar", b"baz"}
    )
) -> None: ...
def f17(
    x: str = "foo"  # Error PYI011 Only simple default values allowed for typed arguments
    + "bar",
) -> None: ...
def f18(
    x: str = b"foo"  # Error PYI011 Only simple default values allowed for typed arguments
    + b"bar",
) -> None: ...
def f19(
    x: object = "foo"  # Error PYI011 Only simple default values allowed for typed arguments
    + 4,
) -> None: ...
def f20(
    x: int = 5
    + 5,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f21(
    x: complex = 3j
    - 3j,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f22(
    x: complex = -42.5j  # Error PYI011 Only simple default values allowed for typed arguments
    + 4.3j,
) -> None: ...
def f23(
    x: bool = True,  # OK
) -> None: ...
def f24(
    x: float = 3.14,  # OK
) -> None: ...
def f25(
    x: float = -3.14,  # OK
) -> None: ...
def f26(
    x: complex = -3.14j,  # OK
) -> None: ...
def f27(
    x: complex = -3 - 3.14j,  # OK
) -> None: ...
def f28(
    x: float = math.tau,  # OK
) -> None: ...
def f29(
    x: float = math.inf,  # OK
) -> None: ...
def f30(
    x: float = -math.inf,  # OK
) -> None: ...
def f31(
    x: float = inf,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f32(
    x: float = np.inf,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f33(
    x: float = math.nan,  # OK
) -> None: ...
def f34(
    x: float = -math.nan,  # Error PYI011 Only simple default values allowed for typed arguments
) -> None: ...
def f35(
    x: complex = math.inf  # Error PYI011 Only simple default values allowed for typed arguments
    + 1j,
) -> None: ...
def f36(
    *,
    x: str = sys.version,  # OK
) -> None: ...
def f37(
    *,
    x: str = ""  # Error PYI011 Only simple default values allowed for typed arguments
    + "",
) -> None: ...



from django import forms


class TestModelForm1(forms.ModelForm):
    class Meta:
        fields = "__all__"


class TestModelForm2(forms.ModelForm):
    class Meta:
        fields = b"__all__"


class TestModelForm3(forms.ModelForm):
    class Meta:
        fields = ["foo"]



from __future__ import annotations

import datetime
from array import array
from dataclasses import dataclass
from uuid import UUID  # TCH003

import attrs
from attrs import frozen


@attrs.define(auto_attribs=True)
class A:
    x: datetime.datetime


@frozen
class B:
    x: array


@dataclass
class C:
    x: UUID



from __future__ import annotations

import datetime
import json


from binascii import hexlify

import requests


from sanic import Sanic
from loguru import Logger

from . import config
from .data import Data



# OK

mocker.patch("module.name", not_lambda)
module_mocker.patch("module.name", not_lambda)
mocker.patch.object(obj, "attr", not_lambda)
module_mocker.patch.object(obj, "attr", not_lambda)

mocker.patch("module.name", return_value=None)
module_mocker.patch("module.name", return_value=None)
mocker.patch.object(obj, "attr", return_value=None)
module_mocker.patch.object(obj, "attr", return_value=None)

mocker.patch("module.name", lambda x, y: x)
module_mocker.patch("module.name", lambda x, y: x)
mocker.patch.object(obj, "attr", lambda x, y: x)
module_mocker.patch.object(obj, "attr", lambda x, y: x)

mocker.patch("module.name", lambda *args: args)
module_mocker.patch("module.name", lambda *args: args)
mocker.patch.object(obj, "attr", lambda *args: args)
module_mocker.patch.object(obj, "attr", lambda *args: args)

mocker.patch("module.name", lambda **kwargs: kwargs)
module_mocker.patch("module.name", lambda **kwargs: kwargs)
mocker.patch.object(obj, "attr", lambda **kwargs: kwargs)
module_mocker.patch.object(obj, "attr", lambda **kwargs: kwargs)

mocker.patch("module.name", lambda x, /, y: x)
module_mocker.patch("module.name", lambda x, /, y: x)
mocker.patch.object(obj, "attr", lambda x, /, y: x)
module_mocker.patch.object(obj, "attr", lambda x, /, y: x)

# Error

mocker.patch("module.name", lambda: None)
module_mocker.patch("module.name", lambda: None)
mocker.patch.object(obj, "attr", lambda: None)
module_mocker.patch.object(obj, "attr", lambda: None)

mocker.patch("module.name", lambda x, y: None)
module_mocker.patch("module.name", lambda x, y: None)
mocker.patch.object(obj, "attr", lambda x, y: None)
module_mocker.patch.object(obj, "attr", lambda x, y: None)

mocker.patch("module.name", lambda *args, **kwargs: None)
module_mocker.patch("module.name", lambda *args, **kwargs: None)
mocker.patch.object(obj, "attr", lambda *args, **kwargs: None)
module_mocker.patch.object(obj, "attr", lambda *args, **kwargs: None)



from subprocess import Popen, call, check_call, check_output, run

# Check different Popen wrappers are checked.
Popen("true", shell=True)
call("true", shell=True)
check_call("true", shell=True)
check_output("true", shell=True)
run("true", shell=True)

# Check values that truthy values are treated as true.
Popen("true", shell=1)
Popen("true", shell=[1])
Popen("true", shell={1: 1})
Popen("true", shell=(1,))

# Check command argument looks unsafe.
var_string = "true"
Popen(var_string, shell=True)
Popen([var_string], shell=True)
Popen([var_string, ""], shell=True)



import datetime
import re
import typing
from dataclasses import dataclass, field
from pathlib import Path
from typing import ClassVar, NamedTuple


def default_function() -> list[int]:
    return []


class ImmutableType(NamedTuple):
    something: int = 8


@dataclass()
class A:
    hidden_mutable_default: list[int] = default_function()
    class_variable: typing.ClassVar[list[int]] = default_function()
    another_class_var: ClassVar[list[int]] = default_function()

    fine_path: Path = Path()
    fine_date: datetime.date = datetime.date(2042, 1, 1)
    fine_timedelta: datetime.timedelta = datetime.timedelta(hours=7)
    fine_tuple: tuple[int] = tuple([1])
    fine_regex: re.Pattern = re.compile(r".*")


DEFAULT_IMMUTABLETYPE_FOR_ALL_DATACLASSES = ImmutableType(40)
DEFAULT_A_FOR_ALL_DATACLASSES = A([1, 2, 3])


@dataclass
class B:
    hidden_mutable_default: list[int] = default_function()
    another_dataclass: A = A()
    not_optimal: ImmutableType = ImmutableType(20)
    good_variant: ImmutableType = DEFAULT_IMMUTABLETYPE_FOR_ALL_DATACLASSES
    okay_variant: A = DEFAULT_A_FOR_ALL_DATACLASSES

    fine_dataclass_function: list[int] = field(default_factory=list)



# PLE1310
"Hello World".strip("Hello")

# PLE1310
"Hello World".strip("Hello")

# PLE1310
"Hello World".strip(u"Hello")

# PLE1310
"Hello World".strip(r"Hello")

# PLE1310
"Hello World".strip("Hello\t")

# PLE1310
"Hello World".strip(r"Hello\t")

# PLE1310
"Hello World".strip("Hello\\")

# PLE1310
"Hello World".strip(r"Hello\\")

# PLE1310
"Hello World".strip("🤣🤣🤣🤣🙃👀😀")

# PLE1310
"Hello World".strip(
    """
there are a lot of characters to strip
"""
)

# PLE1310
"Hello World".strip("can we get a long " \
                    "string of characters to strip " \
                    "please?")

# PLE1310
"Hello World".strip(
    "can we get a long "
    "string of characters to strip "
    "please?"
)

# PLE1310
"Hello World".strip(
    "can \t we get a long"
    "string \t of characters to strip"
    "please?"
)

# PLE1310
"Hello World".strip(
    "abc def"
    "ghi"
)

# PLE1310
u''.strip('http://')

# PLE1310
u''.lstrip('http://')

# PLE1310
b''.rstrip('http://')

# OK
''.strip('Hi')

# OK
''.strip()



import typing


def f(x: typing.List[str]) -> None:
    ...


from typing import List


def f(x: List[str]) -> None:
    ...


import typing as t


def f(x: t.List[str]) -> None:
    ...


from typing import List as IList


def f(x: IList[str]) -> None:
    ...


def f(x: "List[str]") -> None:
    ...


def f(x: r"List[str]") -> None:
    ...


def f(x: "List[str]") -> None:
    ...


def f(x: """List[str]""") -> None:
    ...


def f(x: "Li" "st[str]") -> None:
    ...


def f(x: "List['List[str]']") -> None:
    ...


def f(x: "List['Li' 'st[str]']") -> None:
    ...


def f(x: "Li" "st['List[str]']") -> None:
    ...



import typing as t  # banned
import typing as ty  # banned

import numpy as nmp  # banned
import numpy as npy  # banned
import tensorflow.keras.backend as K  # banned
import torch.nn.functional as F  # banned
from tensorflow.keras import backend as K  # banned
from torch.nn import functional as F  # banned

from typing import Any  # ok

import numpy as np  # ok
import tensorflow as tf  # ok
import torch.nn as nn  # ok
from tensorflow.keras import backend  # ok



import pytest


@pytest.mark.usefixtures("a")
def test_ok():
    pass


@pytest.mark.foo()
def test_ok_another_mark_with_parens():
    pass


@pytest.mark.foo
def test_ok_another_mark_no_parens():
    pass


@pytest.mark.usefixtures()
def test_error_with_parens():
    pass


@pytest.mark.usefixtures
def test_error_no_parens():
    pass



a, b, x, y = 1, 2, 3, 4

# UP031
print('%s %s' % (a, b))

print('%s%s' % (a, b))

print("trivial" % ())

print("%s" % ("simple",))

print("%s" % ("%s" % ("nested",),))

print("%s%% percent" % (15,))

print("%f" % (15,))

print("%.f" % (15,))

print("%.3f" % (15,))

print("%3f" % (15,))

print("%-5f" % (5,))

print("%9f" % (5,))

print("%#o" % (123,))

print("brace {} %s" % (1,))

print(
  "%s" % (
    "trailing comma",
        )
)

print("foo %s " % (x,))

print("%(k)s" % {"k": "v"})

print("%(k)s" % {
    "k": "v",
    "i": "j"
})

print("%(to_list)s" % {"to_list": []})

print("%(k)s" % {"k": "v", "i": 1, "j": []})

print("%(ab)s" % {"a" "b": 1})

print("%(a)s" % {"a"  :  1})

print((
    "foo %s "
    "bar %s" % (x, y)
))

print(
    "foo %(foo)s "
    "bar %(bar)s" % {"foo": x, "bar": y}
)

bar = {"bar": y}
print(
    "foo %(foo)s "
    "bar %(bar)s" % {"foo": x, **bar}
)

print("%s \N{snowman}" % (a,))

print("%(foo)s \N{snowman}" % {"foo": 1})

print(("foo %s " "bar %s") % (x, y))

# Single-value expressions
print('Hello %s' % "World")
print('Hello %s' % f"World")
print('Hello %s (%s)' % bar)
print('Hello %s (%s)' % bar.baz)
print('Hello %s (%s)' % bar['bop'])
print('Hello %(arg)s' % bar)
print('Hello %(arg)s' % bar.baz)
print('Hello %(arg)s' % bar['bop'])

# Hanging modulos
(
    "foo %s "
    "bar %s"
) % (x, y)

(
    "foo %(foo)s "
    "bar %(bar)s"
) % {"foo": x, "bar": y}

(
    """foo %s"""
    % (x,)
)

(
    """
    foo %s
    """
    % (x,)
)



# ok
with open("/abc/tmp", "w") as f:
    f.write("def")

with open("/tmp/abc", "w") as f:
    f.write("def")

with open("/var/tmp/123", "w") as f:
    f.write("def")

with open("/dev/shm/unit/test", "w") as f:
    f.write("def")

# not ok by config
with open("/foo/bar", "w") as f:
    f.write("def")



from abc import abstractmethod
from typing import overload, cast
from typing_extensions import override


###
# Unused arguments on functions.
###
def f(self, x):
    print("Hello, world!")


def f(cls, x):
    print("Hello, world!")


def f(self, x):
    ...


def f(cls, x):
    ...


###
# Unused arguments on lambdas.
###
lambda x: print("Hello, world!")


class C:
    ###
    # Unused arguments.
    ###
    def f(self, x):
        print("Hello, world!")

    def f(self, /, x):
        print("Hello, world!")

    def f(cls, x):
        print("Hello, world!")

    @classmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    def f(x):
        print("Hello, world!")

    ###
    # Unused arguments attached to empty functions (OK).
    ###
    def f(self, x):
        ...

    def f(self, /, x):
        ...

    def f(cls, x):
        ...

    @classmethod
    def f(cls, x):
        ...

    @staticmethod
    def f(cls, x):
        ...

    @staticmethod
    def f(x):
        ...

    def f(self, x):
        """Docstring."""

    def f(self, x):
        """Docstring."""
        ...

    def f(self, x):
        pass

    def f(self, x):
        raise NotImplementedError

    def f(self, x):
        raise NotImplementedError()

    def f(self, x):
        raise NotImplementedError("...")

    def f(self, x):
        raise NotImplemented

    def f(self, x):
        raise NotImplemented()

    def f(self, x):
        raise NotImplemented("...")

    ###
    # Unused functions attached to abstract methods (OK).
    ###
    @abstractmethod
    def f(self, x):
        print("Hello, world!")

    @abstractmethod
    def f(self, /, x):
        print("Hello, world!")

    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @classmethod
    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @abstractmethod
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @abstractmethod
    def f(x):
        print("Hello, world!")

    ###
    # Unused functions attached to overrides (OK).
    ###
    @override
    def f(self, x):
        print("Hello, world!")

    @override
    def f(self, /, x):
        print("Hello, world!")

    @override
    def f(cls, x):
        print("Hello, world!")

    @classmethod
    @override
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @override
    def f(cls, x):
        print("Hello, world!")

    @staticmethod
    @override
    def f(x):
        print("Hello, world!")


###
# Unused arguments attached to overloads (OK).
###
@overload
def f(a: str, b: str) -> str:
    ...


@overload
def f(a: int, b: int) -> str:
    ...


def f(a, b):
    return f"{a}{b}"


###
# Unused arguments on magic methods.
###
class C:
    def __init__(self, x) -> None:
        print("Hello, world!")

    def __str__(self) -> str:
        return "Hello, world!"

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        print("Hello, world!")


###
# Used arguments on chained cast.
###
def f(x: None) -> None:
    _ = cast(Any, _identity)(x=x)



import collections
import datetime as dt
from decimal import Decimal
import logging
import operator
from pathlib import Path
import random
import re
import time
import types
from operator import attrgetter, itemgetter, methodcaller
from types import MappingProxyType


# B006
# Allow immutable literals/calls/comprehensions
def this_is_okay(value=(1, 2, 3)):
    ...


async def and_this_also(value=tuple()):
    pass


def frozenset_also_okay(value=frozenset()):
    pass


def mappingproxytype_okay(
    value=MappingProxyType({}), value2=types.MappingProxyType({})
):
    pass


def re_compile_ok(value=re.compile("foo")):
    pass


def operators_ok(
    v=operator.attrgetter("foo"),
    v2=operator.itemgetter("foo"),
    v3=operator.methodcaller("foo"),
):
    pass


def operators_ok_unqualified(
    v=attrgetter("foo"),
    v2=itemgetter("foo"),
    v3=methodcaller("foo"),
):
    pass


def kwonlyargs_immutable(*, value=()):
    ...


# Flag mutable literals/comprehensions


def this_is_wrong(value=[1, 2, 3]):
    ...


def this_is_also_wrong(value={}):
    ...


def and_this(value=set()):
    ...


def this_too(value=collections.OrderedDict()):
    ...


async def async_this_too(value=collections.defaultdict()):
    ...


def dont_forget_me(value=collections.deque()):
    ...


# N.B. we're also flagging the function call in the comprehension
def list_comprehension_also_not_okay(default=[i**2 for i in range(3)]):
    pass


def dict_comprehension_also_not_okay(default={i: i**2 for i in range(3)}):
    pass


def set_comprehension_also_not_okay(default={i**2 for i in range(3)}):
    pass


def kwonlyargs_mutable(*, value=[]):
    ...


# Recommended approach for mutable defaults
def do_this_instead(value=None):
    if value is None:
        value = set()


# B008
# Flag function calls as default args (including if they are part of a sub-expression)
def in_fact_all_calls_are_wrong(value=time.time()):
    ...


def f(when=dt.datetime.now() + dt.timedelta(days=7)):
    pass


def can_even_catch_lambdas(a=(lambda x: x)()):
    ...


# Recommended approach for function calls as default args
LOGGER = logging.getLogger(__name__)


def do_this_instead_of_calls_in_defaults(logger=LOGGER):
    # That makes it more obvious that this one value is reused.
    ...


# Handle inf/infinity/nan special case
def float_inf_okay(value=float("inf")):
    pass


def float_infinity_okay(value=float("infinity")):
    pass


def float_plus_infinity_okay(value=float("+infinity")):
    pass


def float_minus_inf_okay(value=float("-inf")):
    pass


def float_nan_okay(value=float("nan")):
    pass


def float_minus_NaN_okay(value=float("-NaN")):
    pass


def float_infinity_literal(value=float("1e999")):
    pass


# But don't allow standard floats
def float_int_is_wrong(value=float(3)):
    pass


def float_str_not_inf_or_nan_is_wrong(value=float("3.14")):
    pass


# Allow decimals
def decimal_okay(value=Decimal("0.1")):
    pass

# Allow dates
def date_okay(value=dt.date(2023, 3, 27)):
    pass

# Allow datetimes
def datetime_okay(value=dt.datetime(2023, 3, 27, 13, 51, 59)):
    pass

# Allow timedeltas
def timedelta_okay(value=dt.timedelta(hours=1)):
    pass

# Allow paths
def path_okay(value=Path(".")):
    pass

# B006 and B008
# We should handle arbitrary nesting of these B008.
def nested_combo(a=[float(3), dt.datetime.now()]):
    pass


# Don't flag nested B006 since we can't guarantee that
# it isn't made mutable by the outer operation.
def no_nested_b006(a=map(lambda s: s.upper(), ["a", "b", "c"])):
    pass


# B008-ception.
def nested_b008(a=random.randint(0, dt.datetime.now().year)):
    pass


# Ignore lambda contents since they are evaluated at call time.
def foo(f=lambda x: print(x)):
    f(1)


from collections import abc
from typing import Annotated, Dict, Optional, Sequence, Union, Set


def immutable_annotations(
    a: Sequence[int] | None = [],
    b: Optional[abc.Mapping[int, int]] = {},
    c: Annotated[Union[abc.Set[str], abc.Sized], "annotation"] = set(),
):
    pass


def mutable_annotations(
    a: list[int] | None = [],
    b: Optional[Dict[int, int]] = {},
    c: Annotated[Union[Set[str], abc.Sized], "annotation"] = set(),
):
    pass



# SIM202
if not a != b:
    pass

# SIM202
if not a != (b + c):
    pass

# SIM202
if not (a + b) != c:
    pass

# OK
if not a == b:
    pass

# OK
if a != b:
    pass

# OK
if not a != b:
    raise ValueError()

# OK
def __eq__(self, other):
    return not self != other



import builtins
import os
import posix
from posix import abort
import sys as std_sys
import typing
import typing_extensions
import _thread
import _winapi

import pytest
from pytest import xfail as py_xfail

###
# Errors
###

# if/elif/else
def x(y):
    if not y:
        return 1
    # error


def x(y):
    if not y:
        print()  # error
    else:
        return 2


def x(y):
    if not y:
        return 1

    print()  # error


# for
def x(y):
    for i in range(10):
        if i > 10:
            return i
    # error


def x(y):
    for i in range(10):
        if i > 10:
            return i
    else:
        print()  # error


# A nonexistent function
def func_unknown(x):
    if x > 0:
        return False
    no_such_function()  # error


# A function that does return the control
def func_no_noreturn(x):
    if x > 0:
        return False
    print("", end="")  # error


###
# Non-errors
###

# raise as last return
def x(y):
    if not y:
        return 1
    raise Exception


# last line in while loop
def x(y):
    while i > 0:
        if y > 0:
            return 1
        y += 1


# exclude empty functions
def x(y):
    return None


# return inner with statement
def x(y):
    with y:
        return 1


async def function():
    async with thing as foo:
        return foo.bar


# assert as last return
def x(y):
    if not y:
        return 1
    assert False, "Sanity check"


# return value within loop
def bar1(x, y, z):
    for i in x:
        if i > y:
            break
        return z


def bar3(x, y, z):
    for i in x:
        if i > y:
            if z:
                break
        else:
            return z
        return None


def bar1(x, y, z):
    for i in x:
        if i < y:
            continue
        return z


def bar3(x, y, z):
    for i in x:
        if i < y:
            if z:
                continue
        else:
            return z
        return None


def prompts(self, foo):
    if not foo:
        return []

    for x in foo:
        yield x
        yield x + 1


# Functions that never return
def noreturn_exit(x):
    if x > 0:
        return 1
    exit()


def noreturn_quit(x):
    if x > 0:
        return 1
    quit()


def noreturn_builtins_exit(x):
    if x > 0:
        return 1
    builtins.exit()


def noreturn_builtins_quit(x):
    if x > 0:
        return 1
    builtins.quit()


def noreturn_os__exit(x):
    if x > 0:
        return 1
    os._exit(0)


def noreturn_os_abort(x):
    if x > 0:
        return 1
    os.abort()


def noreturn_posix__exit():
    if x > 0:
        return 1
    posix._exit()


def noreturn_posix_abort():
    if x > 0:
        return 1
    posix.abort()


def noreturn_posix_abort_2():
    if x > 0:
        return 1
    abort()


def noreturn_sys_exit():
    if x > 0:
        return 1
    std_sys.exit(0)


def noreturn_typing_assert_never():
    if x > 0:
        return 1
    typing.assert_never(0)


def noreturn_typing_extensions_assert_never():
    if x > 0:
        return 1
    typing_extensions.assert_never(0)


def noreturn__thread_exit():
    if x > 0:
        return 1
    _thread.exit(0)


def noreturn__winapi_exitprocess():
    if x > 0:
        return 1
    _winapi.ExitProcess(0)


def noreturn_pytest_exit():
    if x > 0:
        return 1
    pytest.exit("oof")


def noreturn_pytest_fail():
    if x > 0:
        return 1
    pytest.fail("oof")


def noreturn_pytest_skip():
    if x > 0:
        return 1
    pytest.skip("oof")


def noreturn_pytest_xfail():
    if x > 0:
        return 1
    pytest.xfail("oof")


def noreturn_pytest_xfail_2():
    if x > 0:
        return 1
    py_xfail("oof")


def nested(values):
    if not values:
        return False

    for value in values:
        print(value)


def while_true():
    while True:
        if y > 0:
            return 1
        y += 1


# match
def x(y):
    match y:
        case 0:
            return 1
        case 1:
            print()  # error


def foo(baz: str) -> str:
    return baz


def end_of_statement():
    def example():
        if True:
            return ""


    def example():
        if True:
            return ""


    def example():
        if True:
            return ""  # type: ignore


    def example():
        if True:
            return ""  ;


    def example():
        if True:
            return "" \
                ;  # type: ignore



import functools


@functools.lru_cache(maxsize=None)
def fixme():
    pass


@other_decorator
@functools.lru_cache(maxsize=None)
def fixme():
    pass


@functools.lru_cache(maxsize=None)
@other_decorator
def fixme():
    pass


@functools.lru_cache()
def ok():
    pass


@functools.lru_cache(maxsize=64)
def ok():
    pass


def user_func():
    pass


@functools.lru_cache(user_func)
def ok():
    pass


def lru_cache(maxsize=None):
    pass


@lru_cache(maxsize=None)
def ok():
    pass



"""Tests to determine standard library import classification.

For typing-only import detection tests, see `TCH002.py`.
"""


def f():
    import os

    x: os


def f():
    import os

    print(os)



import logging
import logging as foo

logging.info("Hello {}".format("World!"))
logging.log(logging.INFO, "Hello {}".format("World!"))
foo.info("Hello {}".format("World!"))
logging.log(logging.INFO, msg="Hello {}".format("World!"))
logging.log(level=logging.INFO, msg="Hello {}".format("World!"))
logging.log(msg="Hello {}".format("World!"), level=logging.INFO)

# Flask support
import flask
from flask import current_app
from flask import current_app as app

flask.current_app.logger.info("Hello {}".format("World!"))
current_app.logger.info("Hello {}".format("World!"))
app.logger.log(logging.INFO, "Hello {}".format("World!"))



def f():  # OK
    return


async def f():  # Too many statements (52/50)
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()
    print()



import sys

if True:
    if sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]


if True:
    if foo:
        pass
    elif sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]

if True:
    if foo:
        pass
    elif sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]
    elif foo:
        cmd = [sys.executable, "-m", "test", "-j0"]

    if foo:
        pass
    elif sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]

    if sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]

    if foo:
        pass
    elif sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]
    else:
        cmd = [sys.executable, "-m", "test", "-j0"]

    if sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]
    else:
        cmd = [sys.executable, "-m", "test", "-j0"]

    if sys.version_info < (3, 3):
        cmd = [sys.executable, "-m", "test.regrtest"]
    elif foo:
        cmd = [sys.executable, "-m", "test", "-j0"]



import pytest


@pytest.mark.usefixtures("a")
def test_something():  # Ok not fixture
    pass


@pytest.mark.usefixtures("a")
@pytest.fixture()
def my_fixture():  # Error before
    return 0


@pytest.fixture()
@pytest.mark.usefixtures("a")
def my_fixture():  # Error after
    return 0



def f():
    for x in y:
        yield x


def g():
    for x, y in z:
        yield (x, y)


def h():
    for x in [1, 2, 3]:
        yield x


def i():
    for x in {x for x in y}:
        yield x


def j():
    for x in (1, 2, 3):
        yield x


def k():
    for x, y in {3: "x", 6: "y"}:
        yield x, y


def f():  # Comment one\n'
    # Comment two\n'
    for x, y in {  # Comment three\n'
        3: "x",  # Comment four\n'
        # Comment five\n'
        6: "y",  # Comment six\n'
    }:  # Comment seven\n'
        # Comment eight\n'
        yield x, y  # Comment nine\n'
        # Comment ten',


def f():
    for x, y in [{3: (3, [44, "long ss"]), 6: "y"}]:
        yield x, y


def f():
    for x, y in z():
        yield x, y

def f():
    def func():
        # This comment is preserved\n'
        for x, y in z():  # Comment one\n'
            # Comment two\n'
            yield x, y  # Comment three\n'
            # Comment four\n'
# Comment\n'
def g():
    print(3)


def f():
    for x in y:
        yield x
    for z in x:
        yield z


def f():
    for x, y in z():
        yield x, y
    x = 1



def bar():
    ...  # OK


def foo():
    """foo"""  # OK


def buzz():
    print("buzz")  # OK, not in stub file


def foo2():
    123  # OK, not in a stub file


def bizz():
    x = 123  # OK, not in a stub file



import pytest


@pytest.mark.parametrize("param", [1, 2, 3])
def test_always_ok(param):
    ...


@pytest.mark.parametrize("param1,param2", [(1, 2), (3, 4)])
def test_csv(param1, param2):
    ...


@pytest.mark.parametrize("   param1,   ,    param2   , ", [(1, 2), (3, 4)])
def test_csv_with_whitespace(param1, param2):
    ...


@pytest.mark.parametrize("param1,param2", [(1, 2), (3, 4)])
def test_csv_bad_quotes(param1, param2):
    ...


@pytest.mark.parametrize(("param1", "param2"), [(1, 2), (3, 4)])
def test_tuple(param1, param2):
    ...


@pytest.mark.parametrize(("param1",), [1, 2, 3])
def test_tuple_one_elem(param1, param2):
    ...


@pytest.mark.parametrize(["param1", "param2"], [(1, 2), (3, 4)])
def test_list(param1, param2):
    ...


@pytest.mark.parametrize(["param1"], [1, 2, 3])
def test_list_one_elem(param1, param2):
    ...


@pytest.mark.parametrize([some_expr, another_expr], [1, 2, 3])
def test_list_expressions(param1, param2):
    ...


@pytest.mark.parametrize([some_expr, "param2"], [1, 2, 3])
def test_list_mixed_expr_literal(param1, param2):
    ...


@pytest.mark.parametrize(("param1, " "param2, " "param3"), [(1, 2, 3), (4, 5, 6)])
def test_implicit_str_concat_with_parens(param1, param2, param3):
    ...


@pytest.mark.parametrize("param1, " "param2, " "param3", [(1, 2, 3), (4, 5, 6)])
def test_implicit_str_concat_no_parens(param1, param2, param3):
    ...


@pytest.mark.parametrize((("param1, " "param2, " "param3")), [(1, 2, 3), (4, 5, 6)])
def test_implicit_str_concat_with_multi_parens(param1, param2, param3):
    ...



if isinstance(a, int) or isinstance(a, float):  # SIM101
    pass

if isinstance(a, (int, float)) or isinstance(a, bool):  # SIM101
    pass

if isinstance(a, int) or isinstance(a, float) or isinstance(b, bool):  # SIM101
    pass

if isinstance(b, bool) or isinstance(a, int) or isinstance(a, float):  # SIM101
    pass

if isinstance(a, int) or isinstance(b, bool) or isinstance(a, float):  # SIM101
    pass

if (isinstance(a, int) or isinstance(a, float)) and isinstance(b, bool):  # SIM101
    pass

if isinstance(a.b, int) or isinstance(a.b, float):  # SIM101
    pass

if isinstance(a(), int) or isinstance(a(), float):  # SIM101
    pass

if isinstance(a, int) and isinstance(b, bool) or isinstance(a, float):
    pass

if isinstance(a, bool) or isinstance(b, str):
    pass

if isinstance(a, int) or isinstance(a.b, float):
    pass


def f():
    # OK
    def isinstance(a, b):
        return False

    if isinstance(a, int) or isinstance(a, float):
        pass



def test_xxx(fixture):  # Ok good param name
    pass


def xxx(_param):  # Ok not a test
    pass


def test_xxx(_fixture):  # Error arg
    pass


def test_xxx(*, _fixture):  # Error kwonly
    pass



x = [2, 3, 1]
list(x)
list(sorted(x))
reversed(sorted(x))
reversed(sorted(x, key=lambda e: e))
reversed(sorted(x, reverse=True))
reversed(sorted(x, key=lambda e: e, reverse=True))
reversed(sorted(x, reverse=True, key=lambda e: e))
reversed(sorted(x, reverse=False))

def reversed(*args, **kwargs):
    return None


reversed(sorted(x, reverse=True))



"""
Test for else-if-used
"""


def ok0():
    """Should not trigger on elif"""
    if 1:
        pass
    elif 2:
        pass


def ok1():
    """If the orelse has more than 1 item in it, shouldn't trigger"""
    if 1:
        pass
    else:
        print()
        if 1:
            pass


def ok2():
    """If the orelse has more than 1 item in it, shouldn't trigger"""
    if 1:
        pass
    else:
        if 1:
            pass
        print()


def not_ok0():
    if 1:
        pass
    else:
        if 2:
            pass


def not_ok1():
    if 1:
        pass
    else:
        if 2:
            pass
        else:
            pass



# office_helper and tests are both first-party,
# but we want tests and experiments to be separated, in that order
from office_helper.core import CoreState
import tests.common.foo as tcf
from tests.common import async_mock_service
from experiments.starry import *
from experiments.weird import varieties
from office_helper.assistants import entity_registry as er



from subprocess import Popen, call, check_call, check_output, run

# Different Popen wrappers are checked.
Popen("true", shell=False)
call("true", shell=False)
check_call("true", shell=False)
check_output("true", shell=False)
run("true", shell=False)

# Values that falsey values are treated as false.
Popen("true", shell=0)
Popen("true", shell=[])
Popen("true", shell={})
Popen("true", shell=None)

# Unknown values are treated as falsey.
Popen("true", shell=True if True else False)

# No value is also caught.
Popen("true")



import sys

if sys.version_info == 2:
    2
else:
    3

if sys.version_info < (3,):
    2
else:
    3

if sys.version_info < (3,0):
    2
else:
    3

if sys.version_info == 3:
    3
else:
    2

if sys.version_info > (3,):
    3
else:
    2

if sys.version_info >= (3,):
    3
else:
    2

from sys import version_info

if version_info > (3,):
    3
else:
    2

if True:
    print(1)
elif sys.version_info < (3,0):
    print(2)
else:
    print(3)

if True:
    print(1)
elif sys.version_info > (3,):
    print(3)
else:
    print(2)

if True:
    print(1)
elif sys.version_info > (3,):
    print(3)

def f():
    if True:
        print(1)
    elif sys.version_info > (3,):
        print(3)

if True:
    print(1)
elif sys.version_info < (3,0):
    print(2)
else:
    print(3)

def f():
    if True:
        print(1)
    elif sys.version_info > (3,):
        print(3)



# Too many args (6/4) for max_args=4
# OK for dummy_variable_rgx ~ "skip_.*"
def f(x, y, z, skip_t, skip_u, skip_v):
    pass


# Too many args (6/4) for max_args=4
# Too many args (6/5) for dummy_variable_rgx ~ "skip_.*"
def f(x, y, z, t, u, v):
    pass



import pytest


@pytest.fixture()
def ok_complex_logic():
    if some_condition:
        resource = acquire_resource()
        yield resource
        resource.release()
        return
    yield None


@pytest.fixture()
def error():
    resource = acquire_resource()
    yield resource



# Error (`from unittest import mock`)
if True:
    import mock

# Error (`from unittest import mock`)
if True:
    import mock, sys

# Error (`from unittest.mock import *`)
if True:
    from mock import *

# Error (`from unittest import mock`)
import mock.mock

# Error (`from unittest import mock`)
import contextlib, mock, sys

# Error (`from unittest import mock`)
import mock, sys
x = "This code should be preserved one line below the mock"

# Error (`from unittest import mock`)
from mock import mock

# Error (keep trailing comma)
from mock import (
    mock,
    a,
    b,
    c,
)
from mock import (
    a,
    b,
    c,
    mock,
)

# Error (avoid trailing comma)
from mock import (
    mock,
    a,
    b,
    c
)
from mock import (
    a,
    b,
    c,
    mock
)
from mock import mock, a, b, c
from mock import a, b, c, mock

if True:
    if False:
        from mock import (
            mock,
            a,
            b,
            c
        )

# OK
import os, io

# Error (`from unittest import mock`)
import mock, mock

# Error (`from unittest import mock as foo`)
import mock as foo

# Error (`from unittest import mock as foo`)
from mock import mock as foo

if True:
    # This should yield multiple, aliased imports.
    import mock as foo, mock as bar, mock

    # This should yield multiple, aliased imports, and preserve `os`.
    import mock as foo, mock as bar, mock, os

if True:
    # This should yield multiple, aliased imports.
    from mock import mock as foo, mock as bar, mock


# OK.
x = mock.Mock()

# Error (`mock.Mock()`).
x = mock.mock.Mock()



class Foo:
    def __str__(self):
        ...

    def __repr__(self):
        ...

    def __len__(self):
        ...

    def __length_hint__(self):
        ...

    def __init__(self):
        ...

    def __del__(self):
        ...

    def __bool__(self):
        ...

    def __bytes__(self):
        ...

    def __format__(self, format_spec):
        ...

    def __contains__(self, item):
        ...

    def __complex__(self):
        ...

    def __int__(self):
        ...

    def __float__(self):
        ...

    def __index__(self):
        ...



# Errors.
nums = [1, 2, 3]
map(lambda x: x + 1, nums)
map(lambda x: str(x), nums)
list(map(lambda x: x * 2, nums))
set(map(lambda x: x % 2 == 0, nums))
dict(map(lambda v: (v, v**2), nums))
map(lambda: "const", nums)
map(lambda _: 3.0, nums)
_ = "".join(map(lambda x: x in nums and "1" or "0", range(123)))
all(map(lambda v: isinstance(v, dict), nums))
filter(func, map(lambda v: v, nums))

# When inside f-string, then the fix should be surrounded by whitespace
_ = f"{set(map(lambda x: x % 2 == 0, nums))}"
_ = f"{dict(map(lambda v: (v, v**2), nums))}"

# Error, but unfixable.
# For simple expressions, this could be: `(x if x else 1 for x in nums)`.
# For more complex expressions, this would differ: `(x + 2 if x else 3 for x in nums)`.
map(lambda x=1: x, nums)

# False negatives.
map(lambda x=2, y=1: x + y, nums, nums)
set(map(lambda x, y: x, nums, nums))


def myfunc(arg1: int, arg2: int = 4):
    return 2 * arg1 + arg2


list(map(myfunc, nums))

[x for x in nums]



# E2502
print("שלום‬")

# E2502
example = "x‏" * 100  #    "‏x" is assigned

# E2502
if access_level != "none‮⁦":  # Check if admin ⁩⁦' and access_level != 'user
    print("You are an admin.")


# E2502
def subtract_funds(account: str, amount: int):
    """Subtract funds from bank account then ⁧"""
    return
    bank[account] -= amount
    return


# OK
print("\u202B\u202E\u05e9\u05DC\u05D5\u05DD\u202C")


# OK
print("\N{RIGHT-TO-LEFT MARK}")

# OK
print("Hello World")



from __future__ import annotations

from typing import Any

from requests import Session

from my_first_party import my_first_party_object

from . import my_local_folder_object
class Thing(object):
  name: str
  def __init__(self, name: str):
    self.name = name



# Shouldn't affect non-union field types.
field1: str

# Should emit for duplicate field types.
field2: str | str  # PYI016: Duplicate union member `str`


# Should emit for union types in arguments.
def func1(arg1: int | int):  # PYI016: Duplicate union member `int`
    print(arg1)


# Should emit for unions in return types.
def func2() -> str | str:  # PYI016: Duplicate union member `str`
    return "my string"


# Should emit in longer unions, even if not directly adjacent.
field3: str | str | int  # PYI016: Duplicate union member `str`
field4: int | int | str  # PYI016: Duplicate union member `int`
field5: str | int | str  # PYI016: Duplicate union member `str`
field6: int | bool | str | int  # PYI016: Duplicate union member `int`

# Shouldn't emit for non-type unions.
field7 = str | str

# Should emit for strangely-bracketed unions.
field8: int | (str | int)  # PYI016: Duplicate union member `int`

# Should handle user brackets when fixing.
field9: int | (int | str)  # PYI016: Duplicate union member `int`
field10: (str | int) | str  # PYI016: Duplicate union member `str`

# Should emit for nested unions.
field11: dict[int | int, str]



from __future__ import annotations


# Test case for https://github.com/charliermarsh/ruff/issues/1552
def f():
    x = 0
    list()[x:]


# Test case for https://github.com/charliermarsh/ruff/issues/2603
def f():
    KeyTupleT = tuple[str, ...]

    keys_checked: set[KeyTupleT] = set()
    return keys_checked



"""foo"""  # OK, not in stub


def foo():
    """foo"""  # OK, doc strings are allowed in non-stubs


class Bar:
    """bar"""  # OK, doc strings are allowed in non-stubs


def bar():
    x = 1
    """foo"""  # OK, not a doc string



###
# Errors
###

"{} {}".format(a, b)

"{1} {0}".format(a, b)

"{x.y}".format(x=z)

"{.x} {.y}".format(a, b)

"{} {}".format(a.b, c.d)

"{}".format(a())

"{}".format(a.b())

"{}".format(a.b().c())

"hello {}!".format(name)

"{}{b}{}".format(a, c, b=b)

"{}".format(0x0)

"{} {}".format(a, b)

"""{} {}""".format(a, b)

"foo{}".format(1)

r"foo{}".format(1)

x = "{a}".format(a=1)

print("foo {} ".format(x))

"{a[b]}".format(a=a)

"{a.a[b]}".format(a=a)

"{}{{}}{}".format(escaped, y)

"{}".format(a)

'({}={{0!e}})'.format(a)

"{[b]}".format(a)

'{[b]}'.format(a)

"""{[b]}""".format(a)

'''{[b]}'''.format(a)

###
# Non-errors
###

# False-negative: RustPython doesn't parse the `\N{snowman}`.
"\N{snowman} {}".format(a)

"{".format(a)

"}".format(a)

"{} {}".format(*a)

"{0} {0}".format(arg)

"{x} {x}".format(arg)

"{x.y} {x.z}".format(arg)

b"{} {}".format(a, b)

"{:{}}".format(x, y)

"{}{}".format(a)

"" "{}".format(a["\\"])

"{}".format(a["b"])

r'"\N{snowman} {}".format(a)'

"{a}" "{b}".format(a=1, b=1)


async def c():
    return "{}".format(await 3)


async def c():
    return "{}".format(1 + await 3)


def d(osname, version, release):
    return"{}-{}.{}".format(osname, version, release)


def e():
    yield"{}".format(1)


assert"{}".format(1)



dict((x, x) for x in range(3))
dict(
    (x, x) for x in range(3)
)
dict(((x, x) for x in range(3)), z=3)
y = f'{dict((x, x) for x in range(3))}'
print(f'Hello {dict((x, x) for x in range(3))} World')



import argparse
from pathlib import Path

parser = argparse.ArgumentParser()
parser.add_argument("target_dir", type=Path)
args = parser.parse_args()
parser.error(f"Target directory {args.target_dir} does not exist")



"""
Violation:

Raising vanilla exception with custom message means it should be customized.
"""

from somewhere import exceptions


def func():
    a = 1
    if a == 1:
        raise Exception("Custom message")

    b = 1
    if b == 1:
        raise Exception


def ignore():
    try:
        a = 1
    except Exception as ex:
        # This is another violation, but this specific analyzer shouldn't care
        raise ex


def anotherfunc():
    a = 1
    if a == 1:
        raise exceptions.Exception("Another except")  # That's fine



"""
Violation:
Reraise without using 'from'
"""


class MyException(Exception):
    pass


def func():
    try:
        a = 1
    except Exception:
        raise MyException()


def func():
    try:
        a = 1
    except Exception:
        if True:
            raise MyException()


def good():
    try:
        a = 1
    except MyException as e:
        raise e  # This is verbose violation, shouldn't trigger no cause
    except Exception:
        raise  # Just re-raising don't need 'from'


def good():
    try:
        from mod import f
    except ImportError:
        def f():
            raise MyException()  # Raising within a new scope is fine



def f():
    # isort: off
    import sys
    import os
    import collections
    # isort: on


def f():
    # ruff: isort: off
    import sys
    import os
    import collections
    # ruff: isort: on


def f():
    import sys
    import os  # isort: skip
    import collections
    import abc


def f():
    import sys
    import os  # isort:skip
    import collections
    import abc



import numpy as npy
import numpy as np
import numpy

# Error
npy.bool
npy.int

if dtype == np.object:
    ...

result = result.select_dtypes([np.byte, np.ubyte, np.short, np.ushort, np.int, np.long])

pdf = pd.DataFrame(
    data=[[1, 2, 3]],
    columns=["a", "b", "c"],
    dtype=numpy.object,
)

_ = arr.astype(np.int)



import hashlib
from hashlib import new as hashlib_new
from hashlib import sha1 as hashlib_sha1

# Invalid

hashlib.new('md5')

hashlib.new('md4', b'test')

hashlib.new(name='md5', data=b'test')

hashlib.new('MD4', data=b'test')

hashlib.new('sha1')

hashlib.new('sha1', data=b'test')

hashlib.new('sha', data=b'test')

hashlib.new(name='SHA', data=b'test')

hashlib.sha(data=b'test')

hashlib.md5()

hashlib_new('sha1')

hashlib_sha1('sha1')

# usedforsecurity arg only available in Python 3.9+
hashlib.new('sha1', usedforsecurity=True)

# Valid

hashlib.new('sha256')

hashlib.new('SHA512')

hashlib.sha256(data=b'test')

# usedforsecurity arg only available in Python 3.9+
hashlib_new(name='sha1', usedforsecurity=False)

# usedforsecurity arg only available in Python 3.9+
hashlib_sha1(name='sha1', usedforsecurity=False)

# usedforsecurity arg only available in Python 3.9+
hashlib.md4(usedforsecurity=False)

# usedforsecurity arg only available in Python 3.9+
hashlib.new(name='sha256', usedforsecurity=False)



from __future__ import annotations

import pathlib
from typing import TYPE_CHECKING

import pydantic
from pydantic import BaseModel

if TYPE_CHECKING:
    import datetime  # TCH004
    from array import array  # TCH004

    import pandas  # TCH004
    import pyproj


class A(pydantic.BaseModel):
    x: datetime.datetime


class B(BaseModel):
    x: pandas.DataFrame


class C(BaseModel):
    x: pathlib.Path


class E:
    pass


class F(E):
    x: pyproj.Transformer


class G(BaseModel):
    x: array



###
# Positive cases
###

# SIM401 (pattern-1)
if key in a_dict:
    var = a_dict[key]
else:
    var = "default1"

# SIM401 (pattern-2)
if key not in a_dict:
    var = "default2"
else:
    var = a_dict[key]

# SIM401 (default with a complex expression)
if key in a_dict:
    var = a_dict[key]
else:
    var = val1 + val2

# SIM401 (complex expression in key)
if keys[idx] in a_dict:
    var = a_dict[keys[idx]]
else:
    var = "default"

# SIM401 (complex expression in dict)
if key in dicts[idx]:
    var = dicts[idx][key]
else:
    var = "default"

# SIM401 (complex expression in var)
if key in a_dict:
    vars[idx] = a_dict[key]
else:
    vars[idx] = "default"

###
# Negative cases
###

# OK (false negative)
if not key in a_dict:
    var = "default"
else:
    var = a_dict[key]

# OK (different dict)
if key in a_dict:
    var = other_dict[key]
else:
    var = "default"

# OK (different key)
if key in a_dict:
    var = a_dict[other_key]
else:
    var = "default"

# OK (different var)
if key in a_dict:
    var = a_dict[key]
else:
    other_var = "default"

# OK (extra vars in body)
if key in a_dict:
    var = a_dict[key]
    var2 = value2
else:
    var = "default"

# OK (extra vars in orelse)
if key in a_dict:
    var = a_dict[key]
else:
    var2 = value2
    var = "default"

# OK (complex default value)
if key in a_dict:
    var = a_dict[key]
else:
    var = foo()

# OK (complex default value)
if key in a_dict:
    var = a_dict[key]
else:
    var = a_dict["fallback"]

# OK (false negative for elif)
if foo():
    pass
elif key in a_dict:
    vars[idx] = a_dict[key]
else:
    vars[idx] = "default"

# OK (false negative for nested else)
if foo():
    pass
else:
    if key in a_dict:
        vars[idx] = a_dict[key]
    else:
        vars[idx] = "default"



class Foo:
    """buzz"""

    pass


if foo:
    """foo"""
    pass


def multi_statement() -> None:
    """This is a function."""
    pass; print("hello")


if foo:
    pass
else:
    """bar"""
    pass


while True:
    pass
else:
    """bar"""
    pass


for _ in range(10):
    pass
else:
    """bar"""
    pass


async for _ in range(10):
    pass
else:
    """bar"""
    pass


def foo() -> None:
    """
    buzz
    """

    pass


async def foo():
    """
    buzz
    """

    pass


try:
    """
    buzz
    """
    pass
except ValueError:
    pass


try:
    bar()
except ValueError:
    """bar"""
    pass


for _ in range(10):
    """buzz"""
    pass

async for _ in range(10):
    """buzz"""
    pass

while cond:
    """buzz"""
    pass


with bar:
    """buzz"""
    pass

async with bar:
    """buzz"""
    pass


def foo() -> None:
    """buzz"""
    pass  # bar


class Foo:
    # bar
    pass


if foo:
    # foo
    pass


class Error(Exception):
    pass


try:
    foo()
except NetworkError:
    pass


def foo() -> None:
    pass



import os as foo
import os.path as foo_p

p = "/foo"

a = foo_p.abspath(p)
aa = foo.chmod(p)
aaa = foo.mkdir(p)
foo.makedirs(p)
foo.rename(p)
foo.replace(p)
foo.rmdir(p)
foo.remove(p)
foo.unlink(p)
foo.getcwd(p)
b = foo_p.exists(p)
bb = foo_p.expanduser(p)
bbb = foo_p.isdir(p)
bbbb = foo_p.isfile(p)
bbbbb = foo_p.islink(p)
foo.readlink(p)
foo.stat(p)
foo_p.isabs(p)
foo_p.join(p)
foo_p.basename(p)
foo_p.dirname(p)
foo_p.samefile(p)
foo_p.splitext(p)



import abc
from abc import abstractmethod

import pytest


@pytest.fixture()
def _patch_something(mocker):  # OK simple
    mocker.patch("some.thing")


@pytest.fixture()
def _patch_something(mocker):  # OK with return
    if something:
        return
    mocker.patch("some.thing")


@pytest.fixture()
def _activate_context():  # OK with yield
    with context:
        yield


class BaseTest:
    @pytest.fixture()
    @abc.abstractmethod
    def my_fixture():  # OK abstract with import abc
        raise NotImplementedError


class BaseTest:
    @pytest.fixture()
    @abstractmethod
    def my_fixture():  # OK abstract with from import
        raise NotImplementedError


@pytest.fixture()
def my_fixture():  # OK ignoring yield from
    yield from some_generator()



@pytest.fixture()
def my_fixture():  # OK ignoring yield value
    yield 1


@pytest.fixture()
def patch_something(mocker):  # Error simple
    mocker.patch("some.thing")


@pytest.fixture()
def activate_context():  # Error with yield
    with context:
        yield



from typing import NamedTuple
import typing

# with complex annotations
MyType = NamedTuple("MyType", [("a", int), ("b", tuple[str, ...])])

# with default values as list
MyType = NamedTuple(
    "MyType",
    [("a", int), ("b", str), ("c", list[bool])],
    defaults=["foo", [True]],
)

# with namespace
MyType = typing.NamedTuple("MyType", [("a", int), ("b", str)])

# too many default values (OK)
MyType = NamedTuple(
    "MyType",
    [("a", int), ("b", str)],
    defaults=[1, "bar", "baz"],
)

# invalid identifiers (OK)
MyType = NamedTuple("MyType", [("x-y", int), ("b", tuple[str, ...])])

# no fields
MyType = typing.NamedTuple("MyType")

# empty fields
MyType = typing.NamedTuple("MyType", [])



# Invalid calls; errors expected.

"{0}" "{1}" "{2}".format(1, 2, 3)

"a {3} complicated {1} string with {0} {2}".format(
    "first", "second", "third", "fourth"
)

'{0}'.format(1)

'{0:x}'.format(30)

x = '{0}'.format(1)

'''{0}\n{1}\n'''.format(1, 2)

x = "foo {0}" \
    "bar {1}".format(1, 2)

("{0}").format(1)

"\N{snowman} {0}".format(1)

'{' '0}'.format(1)

# These will not change because we are waiting for libcst to fix this issue:
# https://github.com/Instagram/LibCST/issues/846
print(
    'foo{0}'
    'bar{1}'.format(1, 2)
)

print(
    'foo{0}'  # ohai\n"
    'bar{1}'.format(1, 2)
)



from pysnmp.hlapi import UsmUserData


insecure = UsmUserData("securityName")  # S509
auth_no_priv = UsmUserData("securityName", "authName")  # S509

less_insecure = UsmUserData("securityName", "authName", "privName")  # OK



def foo():
    pass


# SIM105
try:
    foo()
except ValueError:
    pass

# SIM105
try:
    foo()
except (ValueError, OSError):
    pass

# SIM105
try:
    foo()
except:
    pass

# SIM105
try:
    foo()
except (a.Error, b.Error):
    pass

# OK
try:
    foo()
except ValueError:
    print("foo")
except OSError:
    pass

# OK
try:
    foo()
except ValueError:
    pass
else:
    print("bar")

# OK
try:
    foo()
except ValueError:
    pass
finally:
    print("bar")

# OK
try:
    foo()
    foo()
except ValueError:
    pass

# OK
try:
    for i in range(3):
        foo()
except ValueError:
    pass


def bar():
    # OK
    try:
        return foo()
    except ValueError:
        pass


def with_ellipsis():
    # OK
    try:
        foo()
    except ValueError:
        ...


def with_ellipsis_and_return():
    # OK
    try:
        return foo()
    except ValueError:
        ...


def with_comment():
    try:
        foo()
    except (ValueError, OSError):
        pass  # Trailing comment.



# Test absolute imports
# Violation cases
import xml.dom.minidom
import xml.dom.minidom as wrong
from xml.dom import minidom as wrong
from xml.dom import minidom
from xml.dom.minidom import parseString as wrong  # Ensure ICN001 throws on function import.
from xml.dom.minidom import parseString
from xml.dom.minidom import parse, parseString
from xml.dom.minidom import parse as ps, parseString as wrong

# No ICN001 violations
import xml.dom.minidom as md
from xml.dom import minidom as md
from xml.dom.minidom import parseString as pstr
from xml.dom.minidom import parse, parseString as pstr
from xml.dom.minidom import parse as ps, parseString as pstr


# Test relative imports
from ..xml.dom import minidom as okay  # Ensure config "xml.dom.minidom" doesn't catch relative imports



def bar(): ...  # OK
def foo():
    """foo"""  # OK, strings are handled by another rule

def buzz():
    print("buzz")  # ERROR PYI010

def foo2():
    123  # ERROR PYI010

def bizz():
    x = 123  # ERROR PYI010



from typing import TYPE_CHECKING, Any, ClassVar

import attrs

from ....import unknown
from ..protocol import commands, definitions, responses
from ..server import example
from .. import server
from . import logger, models
from ..protocol.UpperCaseModule import some_function



from foo import (  # Comment on `foo`
    Member as Alias,  # Comment on `Alias`
)

from bar import (  # Comment on `bar`
    Member,  # Comment on `Member`
)

from baz import (  # Comment on `baz`
    Member as Alias  # Comment on `Alias`
)

from bop import (  # Comment on `bop`
    Member  # Comment on `Member`
)



try:
    y = 6 + "7"
except TypeError:
    # RSE102
    raise ValueError()

try:
    x = 1 / 0
except ZeroDivisionError:
    raise

# RSE102
raise TypeError()

# RSE102
raise TypeError ()

# RSE102
raise TypeError \
    ()

# RSE102
raise TypeError(

)

# RSE102
raise TypeError(
    # Hello, world!
)

raise AssertionError

raise AttributeError("test message")



__all__ = "CONST"  # [invalid-all-format]

__all__ = ["Hello"] + {"world"}  # [invalid-all-format]

__all__ += {"world"}  # [invalid-all-format]

__all__ = {"world"} + ["Hello"]  # [invalid-all-format]

__all__ = {"world"} + list(["Hello"])  # [invalid-all-format]

__all__ = list(["Hello"]) + {"world"}  # [invalid-all-format]

__all__ = (x for x in ["Hello", "world"])  # [invalid-all-format]

__all__ = {x for x in ["Hello", "world"]}  # [invalid-all-format]

__all__ = ["Hello"]

__all__ = ("Hello",)

__all__ = ["Hello"] + ("world",)

__all__ = [x for x in ["Hello", "world"]]

__all__ = list(["Hello", "world"])

__all__ = list({"Hello", "world"})

__all__ = list(["Hello"]) + list(["world"])

__all__ = tuple(["Hello"]) + ("world",)



from a.prometheus.metrics import (  # type:ignore[attr-defined]
    TERMINAL_CURRENTLY_RUNNING_TOTAL,
)

from b.prometheus.metrics import (
    TERMINAL_CURRENTLY_RUNNING_TOTAL,  # type:ignore[attr-defined]
)

from c.prometheus.metrics import TERMINAL_CURRENTLY_RUNNING_TOTAL  # type:ignore[attr-defined]

from d.prometheus.metrics import TERMINAL_CURRENTLY_RUNNING_TOTAL, OTHER_RUNNING_TOTAL  # type:ignore[attr-defined]



if a or True:  # SIM222
    pass

if (a or b) or True:  # SIM222
    pass

if a or (b or True):  # SIM222
    pass

if a and True:  # OK
    pass

if True:  # OK
    pass


def validate(self, value):
    return json.loads(value) or True  # OK


if a or f() or b or g() or True:  # OK
    pass

if a or f() or True or g() or b:  # SIM222
    pass

if True or f() or a or g() or b:  # SIM222
    pass

if a or True or f() or b or g():  # SIM222
    pass


if a and f() and b and g() and False:  # OK
    pass

if a and f() and False and g() and b:  # OK
    pass

if False and f() and a and g() and b:  # OK
    pass

if a and False and f() and b and g():  # OK
    pass


a or "" or True  # SIM222

a or "foo" or True or "bar"  # SIM222

a or 0 or True  # SIM222

a or 1 or True or 2  # SIM222

a or 0.0 or True  # SIM222

a or 0.1 or True or 0.2  # SIM222

a or [] or True  # SIM222

a or list([]) or True  # SIM222

a or [1] or True or [2]  # SIM222

a or list([1]) or True or list([2])  # SIM222

a or {} or True  # SIM222

a or dict() or True  # SIM222

a or {1: 1} or True or {2: 2}  # SIM222

a or dict({1: 1}) or True or dict({2: 2})  # SIM222

a or set() or True  # SIM222

a or set(set()) or True  # SIM222

a or {1} or True or {2}  # SIM222

a or set({1}) or True or set({2})  # SIM222

a or () or True  # SIM222

a or tuple(()) or True  # SIM222

a or (1,) or True or (2,)  # SIM222

a or tuple((1,)) or True or tuple((2,))  # SIM222

a or frozenset() or True  # SIM222

a or frozenset(frozenset()) or True  # SIM222

a or frozenset({1}) or True or frozenset({2})  # SIM222

a or frozenset(frozenset({1})) or True or frozenset(frozenset({2}))  # SIM222


# Inside test `a` is simplified.

bool(a or [1] or True or [2])  # SIM222

assert a or [1] or True or [2]  # SIM222

if (a or [1] or True or [2]) and (a or [1] or True or [2]):  # SIM222
    pass

0 if a or [1] or True or [2] else 1  # SIM222

while a or [1] or True or [2]:  # SIM222
    pass

[
    0
    for a in range(10)
    for b in range(10)
    if a or [1] or True or [2]  # SIM222
    if b or [1] or True or [2]  # SIM222
]

{
    0
    for a in range(10)
    for b in range(10)
    if a or [1] or True or [2]  # SIM222
    if b or [1] or True or [2]  # SIM222
}

{
    0: 0
    for a in range(10)
    for b in range(10)
    if a or [1] or True or [2]  # SIM222
    if b or [1] or True or [2]  # SIM222
}

(
    0
    for a in range(10)
    for b in range(10)
    if a or [1] or True or [2]  # SIM222
    if b or [1] or True or [2]  # SIM222
)

# Outside test `a` is not simplified.

a or [1] or True or [2]  # SIM222

if (a or [1] or True or [2]) == (a or [1]):  # SIM222
    pass

if f(a or [1] or True or [2]):  # SIM222
    pass



# OK
import other
import other.example
from other import example

# TID252
from . import sibling
from .sibling import example
from .. import parent
from ..parent import example
from ... import grandparent
from ...grandparent import example
from  .parent import hello
from .\
    parent import \
        hello_world
from \
    ..parent\
    import \
    world_hello
from ..... import ultragrantparent
from ...... import ultragrantparent
from ....... import ultragrantparent
from ......... import ultragrantparent
from ........................... import ultragrantparent
from .....parent import ultragrantparent
from .........parent import ultragrantparent
from ...........................parent import ultragrantparent



key in obj.keys()  # SIM118

foo["bar"] in obj.keys()  # SIM118

foo['bar'] in obj.keys()  # SIM118

foo() in obj.keys()  # SIM118

for key in obj.keys():  # SIM118
    pass

for key in list(obj.keys()):
    if some_property(key):
        del obj[key]

[k for k in obj.keys()]  # SIM118

{k for k in obj.keys()}  # SIM118

{k: k for k in obj.keys()}  # SIM118

(k for k in obj.keys())  # SIM118

key in (obj or {}).keys()  # SIM118



def func_all_spaces():
    # No error
    print("spaces")

def func_tabs():
    # No error
	print("tabs")

def func_mixed_start_with_tab():
    # E101
	  print("mixed starts with tab")

def func_mixed_start_with_space():
    # E101
 	  	  	   print("mixed starts with space")

def xyz():
    # E101
   	print("xyz");



# SIM910
{}.get(key, None)

# SIM910
{}.get("key", None)

# OK
{}.get(key)

# OK
{}.get("key")

# OK
{}.get(key, False)

# OK
{}.get("key", False)

# SIM910
if a := {}.get(key, None):
    pass

# SIM910
a = {}.get(key, None)

# SIM910
({}).get(key, None)



import itertools
from itertools import groupby

shoppers = ["Jane", "Joe", "Sarah"]
items = [
    ("lettuce", "greens"),
    ("tomatoes", "greens"),
    ("cucumber", "greens"),
    ("chicken breast", "meats & fish"),
    ("salmon", "meats & fish"),
    ("ice cream", "frozen items"),
]

carts = {shopper: [] for shopper in shoppers}


def collect_shop_items(shopper, items):
    # Imagine this an expensive database query or calculation that is
    # advantageous to batch.
    carts[shopper] += items


# Invoking the `groupby` function directly
for _section, section_items in groupby(items, key=lambda p: p[1]):
    for shopper in shoppers:
        shopper = shopper.title()
        collect_shop_items(shopper, section_items)  # B031
    # We're outside the nested loop and used the group again.
    collect_shop_items(shopper, section_items)  # B031

for _section, section_items in groupby(items, key=lambda p: p[1]):
    collect_shop_items("Jane", section_items)
    collect_shop_items("Joe", section_items)  # B031


# Make sure to detect in other loop constructs as well - `while` loop
for _section, section_items in groupby(items, key=lambda p: p[1]):
    countdown = 3
    while countdown > 0:
        collect_shop_items(shopper, section_items)  # B031
        countdown -= 1

# Make sure to detect in other loop constructs as well - `list` comprehension
collection = []
for _section, section_items in groupby(items, key=lambda p: p[1]):
    collection.append([list(section_items) for _ in range(3)])  # B031

unique_items = set()
another_set = set()
for _section, section_items in groupby(items, key=lambda p: p[1]):
    # For nested loops, it should not flag the usage of the name
    for item in section_items:
        unique_items.add(item)

    # But it should be detected when used again
    for item in section_items:  # B031
        another_set.add(item)

for _section, section_items in groupby(items, key=lambda p: p[1]):
    # Variable has been overridden, skip checking
    section_items = list(unique_items)
    collect_shop_items("Jane", section_items)
    collect_shop_items("Jane", section_items)

for _section, section_items in groupby(items, key=lambda p: p[1]):
    # Variable has been overridden, skip checking
    # Not a realistic situation, just for testing purpose
    (section_items := list(unique_items))
    collect_shop_items("Jane", section_items)
    collect_shop_items("Jane", section_items)

for _section, section_items in groupby(items, key=lambda p: p[1]):
    # This is ok
    collect_shop_items("Jane", section_items)

# Invocation via the `itertools` module
for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    for shopper in shoppers:
        collect_shop_items(shopper, section_items)  # B031

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    _ = [collect_shop_items(shopper, section_items) for shopper in shoppers]  # B031

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    # The variable is overridden, skip checking.
    _ = [_ for section_items in range(3)]
    _ = [collect_shop_items(shopper, section_items) for shopper in shoppers]

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    _ = [item for item in section_items]

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    # The iterator is being used for the second time.
    _ = [(item1, item2) for item1 in section_items for item2 in section_items]  # B031

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    if _section == "greens":
        collect_shop_items(shopper, section_items)
    else:
        collect_shop_items(shopper, section_items)
        collect_shop_items(shopper, section_items)  # B031

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    # Mutually exclusive branches shouldn't trigger the warning
    if _section == "greens":
        collect_shop_items(shopper, section_items)
        if _section == "greens":
            collect_shop_items(shopper, section_items)  # B031
        elif _section == "frozen items":
            collect_shop_items(shopper, section_items)  # B031
        else:
            collect_shop_items(shopper, section_items)  # B031
        collect_shop_items(shopper, section_items)  # B031
    elif _section == "frozen items":
        # Mix `match` and `if` statements
        match shopper:
            case "Jane":
                collect_shop_items(shopper, section_items)
                if _section == "fourth":
                    collect_shop_items(shopper, section_items)  # B031
            case _:
                collect_shop_items(shopper, section_items)
    else:
        collect_shop_items(shopper, section_items)
    # Now, it should detect
    collect_shop_items(shopper, section_items)  # B031

for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    # Mutually exclusive branches shouldn't trigger the warning
    match _section:
        case "greens":
            collect_shop_items(shopper, section_items)
            match shopper:
                case "Jane":
                    collect_shop_items(shopper, section_items)  # B031
                case _:
                    collect_shop_items(shopper, section_items)  # B031
        case "frozen items":
            collect_shop_items(shopper, section_items)
            collect_shop_items(shopper, section_items)  # B031
        case _:
            collect_shop_items(shopper, section_items)
    # Now, it should detect
    collect_shop_items(shopper, section_items)  # B031

for group in groupby(items, key=lambda p: p[1]):
    # This is bad, but not detected currently
    collect_shop_items("Jane", group[1])
    collect_shop_items("Joe", group[1])


# https://github.com/charliermarsh/ruff/issues/4050
for _section, section_items in itertools.groupby(items, key=lambda p: p[1]):
    if _section == "greens":
        for item in section_items:
            collect_shop_items(shopper, item)
    elif _section == "frozen items":
        _ = [item for item in section_items]
    else:
        collect_shop_items(shopper, section_items)

#  Make sure we ignore - but don't fail on more complicated invocations
for _key, (_value1, _value2) in groupby(
    [("a", (1, 2)), ("b", (3, 4)), ("a", (5, 6))], key=lambda p: p[1]
):
    collect_shop_items("Jane", group[1])
    collect_shop_items("Joe", group[1])

#  Make sure we ignore - but don't fail on more complicated invocations
for (_key1, _key2), (_value1, _value2) in groupby(
    [(("a", "a"), (1, 2)), (("b", "b"), (3, 4)), (("a", "a"), (5, 6))],
    key=lambda p: p[1],
):
    collect_shop_items("Jane", group[1])
    collect_shop_items("Joe", group[1])


# Let's redefine the `groupby` function to make sure we pick up the correct one.
# NOTE: This should always be at the end of the file.
def groupby(data, key=None):
    pass


for name, group in groupby(items):
    collect_shop_items("Jane", items)
    # This shouldn't be flagged as the `groupby` function is different
    collect_shop_items("Joe", items)



from django.db.models.signals import pre_save
from django.dispatch import receiver
from myapp.models import MyModel

test_decorator = lambda func: lambda *args, **kwargs: func(*args, **kwargs)


@receiver(pre_save, sender=MyModel)
@test_decorator
def correct_pre_save_handler():
    pass


@test_decorator
@receiver(pre_save, sender=MyModel)
def incorrect_pre_save_handler():
    pass


@receiver(pre_save, sender=MyModel)
@receiver(pre_save, sender=MyModel)
@test_decorator
def correct_multiple():
    pass


@receiver(pre_save, sender=MyModel)
@receiver(pre_save, sender=MyModel)
def correct_multiple():
    pass


@receiver(pre_save, sender=MyModel)
@test_decorator
@receiver(pre_save, sender=MyModel)
def incorrect_multiple():
    pass



import pandas as pd

x = pd.DataFrame()

x.drop(["a"], axis=1, inplace=True)

x.drop(["a"], axis=1, inplace=True)

x.drop(
    inplace=True,
    columns=["a"],
    axis=1,
)

if True:
    x.drop(
        inplace=True,
        columns=["a"],
        axis=1,
    )

x.drop(["a"], axis=1, **kwargs, inplace=True)
x.drop(["a"], axis=1, inplace=True, **kwargs)
f(x.drop(["a"], axis=1, inplace=True))

x.apply(lambda x: x.sort_values('a', inplace=True))
import torch
torch.m.ReLU(inplace=True) # safe because this isn't a pandas call



def f(x, y, z):
    """Do something.

    Args:
        x: the value
            with a hanging indent

    Returns:
        the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x:
            The whole thing has a hanging indent.

    Returns:
        the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x:
            The whole thing has a hanging indent.

    Returns: the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x: the value def
            ghi

    Returns:
        the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x: the value
        z: A final argument

    Returns:
        the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x: the value
        z: A final argument

    Returns: the value
    """
    return x


def f(x, y, z):
    """Do something.

    Args:
        x: the value
        z: A final argument
    """
    return x


def f(x, *args, **kwargs):
    """Do something.

    Args:
        x: the value
        *args: variable arguments
        **kwargs: keyword arguments
    """
    return x


def f(x, *args, **kwargs):
    """Do something.

    Args:
        *args: variable arguments
        **kwargs: keyword arguments
    """
    return x


def f(x, *args, **kwargs):
    """Do something.

    Args:
        x: the value
        **kwargs: keyword arguments
    """
    return x


def f(x, *, y, z):
    """Do something.

    Args:
        x: some first value

    Keyword Args:
        y (int): the other value
        z (int): the last value

    """
    return x, y, z


class Test:
    def f(self, /, arg1: int) -> None:
        """
        Some beauty description.

        Args:
            arg1: some description of arg
        """
